@@ -1,8 +1,8 @@
 /*
  SPDX-License-Identifier: MIT
 
- Parson 1.2.1 ( http://kgabis.github.com/parson/ )
- Copyright (c) 2012 - 2021 Krzysztof Gabis
+ Parson 1.5.2 (https://github.com/kgabis/parson)
+ Copyright (c) 2012 - 2023 Krzysztof Gabis
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -31,8 +31,8 @@
 #include "parson.h"
 
 #define PARSON_IMPL_VERSION_MAJOR 1
-#define PARSON_IMPL_VERSION_MINOR 2
-#define PARSON_IMPL_VERSION_PATCH 1
+#define PARSON_IMPL_VERSION_MINOR 5
+#define PARSON_IMPL_VERSION_PATCH 2
 
 #if (PARSON_VERSION_MAJOR != PARSON_IMPL_VERSION_MAJOR)\
 || (PARSON_VERSION_MINOR != PARSON_IMPL_VERSION_MINOR)\
@@ -63,8 +63,17 @@
 #define STARTING_CAPACITY 16
 #define MAX_NESTING       2048
 
-#define FLOAT_FORMAT "%1.17g" /* do not increase precision without incresing NUM_BUF_SIZE */
-#define NUM_BUF_SIZE 64 /* double printed with "%1.17g" shouldn't be longer than 25 bytes so let's be paranoid and use 64 */
+#ifndef PARSON_DEFAULT_FLOAT_FORMAT
+#define PARSON_DEFAULT_FLOAT_FORMAT "%1.17g" /* do not increase precision without incresing NUM_BUF_SIZE */
+#endif
+
+#ifndef PARSON_NUM_BUF_SIZE
+#define PARSON_NUM_BUF_SIZE 64 /* double printed with "%1.17g" shouldn't be longer than 25 bytes so let's be paranoid and use 64 */
+#endif
+
+#ifndef PARSON_INDENT_STR
+#define PARSON_INDENT_STR "    "
+#endif
 
 #define SIZEOF_TOKEN(a)       (sizeof(a) - 1)
 #define SKIP_CHAR(str)        ((*str)++)
@@ -87,6 +96,10 @@ static JSON_Free_Function parson_free = free;
 
 static int parson_escape_slashes = 1;
 
+static char *parson_float_format = NULL;
+
+static JSON_Number_Serialization_Function parson_number_serialization_function = NULL;
+
 #define IS_CONT(b) (((unsigned char)(b) & 0xC0) == 0x80) /* is utf-8 continuation byte */
 
 typedef int parson_bool_t;
@@ -185,8 +198,6 @@ static JSON_Value *  parse_value(const char **string, size_t nesting);
 /* Serialization */
 static int json_serialize_to_buffer_r(const JSON_Value *value, char *buf, int level, parson_bool_t is_pretty, char *num_buf);
 static int json_serialize_string(const char *string, size_t len, char *buf);
-static int append_indent(char *buf, int level);
-static int append_string(char *buf, const char *string);
 
 /* Various */
 static char * read_file(const char * filename) {
@@ -430,7 +441,7 @@ static JSON_Status json_object_init(JSON_Object *object, size_t capacity) {
 
     object->count = 0;
     object->cell_capacity = capacity;
-    object->item_capacity = (unsigned int)(capacity * 0.7f);
+    object->item_capacity = (unsigned int)(capacity * 7/10);
 
     if (capacity == 0) {
         return JSONSuccess;
@@ -589,7 +600,7 @@ static JSON_Status json_object_add(JSON_Object *object, char *name, JSON_Value *
 static JSON_Value * json_object_getn_value(const JSON_Object *object, const char *name, size_t name_len) {
     unsigned long hash = 0;
     parson_bool_t found = PARSON_FALSE;
-    unsigned long cell_ix = 0;
+    size_t cell_ix = 0;
     size_t item_ix = 0;
     if (!object || !name) {
         return NULL;
@@ -973,6 +984,9 @@ static JSON_Value * parse_object_value(const char **string, size_t nesting) {
         }
         SKIP_CHAR(string);
         SKIP_WHITESPACES(string);
+        if (**string == '}') {
+            break;
+        }
     }
     SKIP_WHITESPACES(string);
     if (**string != '}') {
@@ -1018,6 +1032,9 @@ static JSON_Value * parse_array_value(const char **string, size_t nesting) {
         }
         SKIP_CHAR(string);
         SKIP_WHITESPACES(string);
+        if (**string == ']') {
+            break;
+        }
     }
     SKIP_WHITESPACES(string);
     if (**string != ']' || /* Trim array after parsing is over */
@@ -1082,15 +1099,27 @@ static JSON_Value * parse_null_value(const char **string) {
 }
 
 /* Serialization */
-#define APPEND_STRING(str) do { written = append_string(buf, (str));\
-                                if (written < 0) { return -1; }\
-                                if (buf != NULL) { buf += written; }\
-                                written_total += written; } while(0)
 
-#define APPEND_INDENT(level) do { written = append_indent(buf, (level));\
-                                  if (written < 0) { return -1; }\
-                                  if (buf != NULL) { buf += written; }\
-                                  written_total += written; } while(0)
+/*  APPEND_STRING() is only called on string literals.
+    It's a bit hacky because it makes plenty of assumptions about the external state
+    and should eventually be tidied up into a function (same goes for APPEND_INDENT)
+ */
+#define APPEND_STRING(str) do {\
+                                written = SIZEOF_TOKEN((str));\
+                                if (buf != NULL) {\
+                                    memcpy(buf, (str), written);\
+                                    buf[written] = '\0';\
+                                    buf += written;\
+                                }\
+                                written_total += written;\
+                            } while (0)
+
+#define APPEND_INDENT(level) do {\
+                                int level_i = 0;\
+                                for (level_i = 0; level_i < (level); level_i++) {\
+                                    APPEND_STRING(PARSON_INDENT_STR);\
+                                }\
+                            } while (0)
 
 static int json_serialize_to_buffer_r(const JSON_Value *value, char *buf, int level, parson_bool_t is_pretty, char *num_buf)
 {
@@ -1212,7 +1241,13 @@ static int json_serialize_to_buffer_r(const JSON_Value *value, char *buf, int le
             if (buf != NULL) {
                 num_buf = buf;
             }
-            written = sprintf(num_buf, FLOAT_FORMAT, num);
+            if (parson_number_serialization_function) {
+                written = parson_number_serialization_function(num, num_buf);
+            } else if (parson_float_format) {
+                written = snprintf(num_buf, PARSON_NUM_BUF_SIZE, parson_float_format, num);
+            } else {
+                written = snprintf(num_buf, PARSON_NUM_BUF_SIZE, PARSON_DEFAULT_FLOAT_FORMAT, num);
+            }
             if (written < 0) {
                 return -1;
             }
@@ -1298,22 +1333,6 @@ static int json_serialize_string(const char *string, size_t len, char *buf) {
     return written_total;
 }
 
-static int append_indent(char *buf, int level) {
-    int i;
-    int written = -1, written_total = 0;
-    for (i = 0; i < level; i++) {
-        APPEND_STRING("    ");
-    }
-    return written_total;
-}
-
-static int append_string(char *buf, const char *string) {
-    if (buf == NULL) {
-        return (int)strlen(string);
-    }
-    return sprintf(buf, "%s", string);
-}
-
 #undef APPEND_STRING
 #undef APPEND_INDENT
 
@@ -1757,7 +1776,7 @@ JSON_Value * json_value_deep_copy(const JSON_Value *value) {
 }
 
 size_t json_serialization_size(const JSON_Value *value) {
-    char num_buf[NUM_BUF_SIZE]; /* recursively allocating buffer on stack is a bad idea, so let's do it only once */
+    char num_buf[PARSON_NUM_BUF_SIZE]; /* recursively allocating buffer on stack is a bad idea, so let's do it only once */
     int res = json_serialize_to_buffer_r(value, NULL, 0, PARSON_FALSE, num_buf);
     return res < 0 ? 0 : (size_t)(res) + 1;
 }
@@ -1817,7 +1836,7 @@ char * json_serialize_to_string(const JSON_Value *value) {
 }
 
 size_t json_serialization_size_pretty(const JSON_Value *value) {
-    char num_buf[NUM_BUF_SIZE]; /* recursively allocating buffer on stack is a bad idea, so let's do it only once */
+    char num_buf[PARSON_NUM_BUF_SIZE]; /* recursively allocating buffer on stack is a bad idea, so let's do it only once */
     int res = json_serialize_to_buffer_r(value, NULL, 0, PARSON_TRUE, num_buf);
     return res < 0 ? 0 : (size_t)(res) + 1;
 }
@@ -2136,7 +2155,7 @@ JSON_Status json_object_dotset_value(JSON_Object *object, const char *name, JSON
     JSON_Status status = JSONFailure;
     size_t name_len = 0;
     char *name_copy = NULL;
-    
+
     if (object == NULL || name == NULL || value == NULL) {
         return JSONFailure;
     }
@@ -2255,9 +2274,15 @@ JSON_Status json_object_clear(JSON_Object *object) {
     }
     for (i = 0; i < json_object_get_count(object); i++) {
         parson_free(object->names[i]);
+        object->names[i] = NULL;
+
         json_value_free(object->values[i]);
+        object->values[i] = NULL;
     }
     object->count = 0;
+    for (i = 0; i < object->cell_capacity; i++) {
+        object->cells[i] = OBJECT_INVALID_IX;
+    }
     return JSONSuccess;
 }
 
@@ -2422,3 +2447,19 @@ void json_set_allocation_functions(JSON_Malloc_Function malloc_fun, JSON_Free_Fu
 void json_set_escape_slashes(int escape_slashes) {
     parson_escape_slashes = escape_slashes;
 }
+
+void json_set_float_serialization_format(const char *format) {
+    if (parson_float_format) {
+        parson_free(parson_float_format);
+        parson_float_format = NULL;
+    }
+    if (!format) {
+        parson_float_format = NULL;
+        return;
+    }
+    parson_float_format = parson_strdup(format);
+}
+
+void json_set_number_serialization_function(JSON_Number_Serialization_Function func) {
+    parson_number_serialization_function = func;
+}
