@@ -1,8 +1,8 @@
 /*
  SPDX-License-Identifier: MIT
 
- Parson ( http://kgabis.github.com/parson/ )
- Copyright (c) 2012 - 2021 Krzysztof Gabis
+ Parson (https://github.com/kgabis/parson)
+ Copyright (c) 2012 - 2023 Krzysztof Gabis
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -62,6 +62,9 @@ void test_suite_10(void); /* Testing for memory leaks */
 void test_suite_11(void); /* Additional things that require testing */
 void test_memory_leaks(void);
 void test_failing_allocations(void);
+void test_custom_number_format(void);
+void test_custom_number_serialization_function(void);
+void test_object_clear(void);
 
 void print_commits_info(const char *username, const char *repo);
 void persistence_example(void);
@@ -97,6 +100,9 @@ int main(int argc, char *argv[]) {
 #else
 int tests_main(int argc, char *argv[]);
 int tests_main(int argc, char *argv[]) {
+#endif
+#if 0 /* unconfuse xcode */
+}
 #endif
     /* Example functions from readme file:      */
     /* print_commits_info("torvalds", "linux"); */
@@ -127,6 +133,9 @@ int tests_main(int argc, char *argv[]) {
     test_suite_11();
     test_memory_leaks();
     test_failing_allocations();
+    test_custom_number_format();
+    test_custom_number_serialization_function();
+    test_object_clear();
 
     printf("Tests failed: %d\n", g_tests_failed);
     printf("Tests passed: %d\n", g_tests_passed);
@@ -302,6 +311,8 @@ void test_suite_3(void) {
     TEST(json_parse_string("false") != NULL);
     TEST(json_parse_string("\"string\"") != NULL);
     TEST(json_parse_string("123") != NULL);
+    TEST(json_parse_string("[\"lorem\",]") != NULL);
+    TEST(json_parse_string("{\"lorem\":\"ipsum\",}") != NULL);
 
     /* Test UTF-16 parsing */
     TEST(STREQ(json_string(json_parse_string("\"\\u0024x\"")), "$x"));
@@ -313,9 +324,9 @@ void test_suite_3(void) {
     g_malloc_count = 0;
     TEST(json_parse_string(NULL) == NULL);
     TEST(json_parse_string("") == NULL); /* empty string */
-    TEST(json_parse_string("[\"lorem\",]") == NULL);
-    TEST(json_parse_string("{\"lorem\":\"ipsum\",}") == NULL);
     TEST(json_parse_string("{lorem:ipsum}") == NULL);
+    TEST(json_parse_string("{\"lorem\":\"ipsum\",]") == NULL);
+    TEST(json_parse_string("{\"lorem\":\"ipsum\",,}") == NULL);
     TEST(json_parse_string("[,]") == NULL);
     TEST(json_parse_string("[,") == NULL);
     TEST(json_parse_string("[") == NULL);
@@ -358,7 +369,7 @@ void test_suite_3(void) {
     TEST(g_malloc_count == 0);
 }
 
-void test_suite_4() {
+void test_suite_4(void) {
     const char *filename = "test_2.txt";
     JSON_Value *a = NULL, *a_copy = NULL;
     a = json_parse_file(get_file_path(filename));
@@ -592,7 +603,7 @@ void test_suite_10(void) {
     TEST(g_malloc_count == 0);
 }
 
-void test_suite_11() {
+void test_suite_11(void) {
     const char * array_with_slashes = "[\"a/b/c\"]";
     const char * array_with_escaped_slashes = "[\"a\\/b\\/c\"]";
     char *serialized = NULL;
@@ -610,7 +621,7 @@ void test_suite_11() {
     TEST(STREQ(array_with_escaped_slashes, serialized));
 }
 
-void test_memory_leaks() {
+void test_memory_leaks(void) {
     g_malloc_count = 0;
 
     TEST(json_object_set_string(NULL, "lorem", "ipsum") == JSONFailure);
@@ -623,7 +634,7 @@ void test_memory_leaks() {
     TEST(g_malloc_count == 0);
 }
 
-void test_failing_allocations() {
+void test_failing_allocations(void) {
     const char *filename = "test_2.txt";
     JSON_Value *root_value = NULL;
     JSON_Object *root_object = NULL;
@@ -682,10 +693,63 @@ void test_failing_allocations() {
         }
     }
 
-    json_set_allocation_functions(NULL, NULL);
+    json_set_allocation_functions(counted_malloc, counted_free);
     printf("OK (tested %d failing allocations)\n", n - 1);
     g_tests_passed++;
+}
 
+void test_custom_number_format(void) {
+    g_malloc_count = 0;
+    {
+        char *serialized = NULL;
+        JSON_Value *val = json_value_init_number(0.6);
+        json_set_float_serialization_format("%.1f");
+        serialized = json_serialize_to_string(val);
+        json_set_float_serialization_format(NULL);
+        TEST(STREQ(serialized, "0.6"));
+        json_free_serialized_string(serialized);
+        json_value_free(val);
+    }
+    TEST(g_malloc_count == 0);
+}
+
+static int custom_serialization_func_called = 0;
+static int custom_serialization_func(double num, char *buf) {
+    char num_buf[32];
+    custom_serialization_func_called = 1;
+    if (buf == NULL)
+        buf = num_buf;
+    return sprintf(buf, "%.1f", num);
+}
+
+void test_custom_number_serialization_function(void) {
+    g_malloc_count = 0;
+    {
+        /* We just test that custom_serialization_func() gets called, not it's performance */
+        char *serialized = NULL;
+        JSON_Value *val = json_value_init_number(0.6);
+        json_set_number_serialization_function(custom_serialization_func);
+        serialized = json_serialize_to_string(val);
+        TEST(STREQ(serialized, "0.6"));
+        TEST(custom_serialization_func_called);
+        json_set_number_serialization_function(NULL);
+        json_free_serialized_string(serialized);
+        json_value_free(val);
+    }
+    TEST(g_malloc_count == 0);
+}
+
+void test_object_clear(void) {
+    g_malloc_count = 0;
+    {
+        JSON_Value *val = json_value_init_object();
+        JSON_Object *obj = json_value_get_object(val);
+        json_object_set_string(obj, "foo", "bar");
+        json_object_clear(obj);
+        TEST(json_object_get_value(obj, "foo") == NULL);
+        json_value_free(val);
+    }
+    TEST(g_malloc_count == 0);
 }
 
 void print_commits_info(const char *username, const char *repo) {
