@@ -51,6 +51,7 @@ bool catalog_register_setup(DatabaseCatalog *catalog,
 							const char *target_pg_uri,
 							const char *snapshot,
 							uint64_t splitTablesLargerThanBytes,
+							int splitMaxParts,
 							const char *filters);
 
 bool catalog_setup_replication(DatabaseCatalog *catalog,
@@ -167,6 +168,25 @@ bool catalog_s_table_stats_fetch(SQLiteQuery *query);
 bool catalog_count_objects(DatabaseCatalog *catalog, CatalogCounts *count);
 bool catalog_count_fetch(SQLiteQuery *query);
 
+/*
+ * Materialized views
+ */
+typedef struct CatalogMatView
+{
+	uint32_t oid;
+	char nspname[PG_NAMEDATALEN];
+	char relname[PG_NAMEDATALEN];
+	char restoreListName[RESTORE_LIST_NAMEDATALEN];
+	bool excludeData;
+} CatalogMatView;
+
+bool catalog_add_s_matview(DatabaseCatalog *catalog, SourceTable *table);
+
+bool catalog_lookup_s_matview_by_oid(DatabaseCatalog *catalog,
+									 CatalogMatView *result,
+									 uint32_t oid);
+bool catalog_s_matview_fetch(SQLiteQuery *query);
+
 /*
  * Tables and their attributes and parts (COPY partitioning).
  */
@@ -201,6 +221,10 @@ bool catalog_iter_s_table_nopk(DatabaseCatalog *catalog,
 							   void *context,
 							   SourceTableIterFun *callback);
 
+bool catalog_iter_s_table_generated_columns(DatabaseCatalog *catalog,
+											void *context,
+											SourceTableIterFun *callback);
+
 typedef struct SourceTableIterator
 {
 	DatabaseCatalog *catalog;
@@ -210,6 +234,7 @@ typedef struct SourceTableIterator
 
 bool catalog_iter_s_table_init(SourceTableIterator *iter);
 bool catalog_iter_s_table_nopk_init(SourceTableIterator *iter);
+bool catalog_iter_s_table_generated_columns_init(SourceTableIterator *iter);
 bool catalog_iter_s_table_next(SourceTableIterator *iter);
 bool catalog_iter_s_table_finish(SourceTableIterator *iter);
 
@@ -602,6 +627,16 @@ bool catalog_count_summary_done(DatabaseCatalog *catalog,
 bool catalog_count_summary_done_fetch(SQLiteQuery *query);
 
 
+/*
+ * Logical decoding
+ */
+bool catalog_add_timeline_history(DatabaseCatalog *catalog,
+								  TimelineHistoryEntry *entry);
+bool catalog_lookup_timeline_history(DatabaseCatalog *catalog,
+									 int tli,
+									 TimelineHistoryEntry *entry);
+bool catalog_timeline_history_fetch(SQLiteQuery *query);
+
 /*
  * Internal tooling for catalogs management
  */
