@@ -34,6 +34,8 @@
 	"  --restore-jobs                Number of concurrent jobs for pg_restore\n" \
 	"  --large-objects-jobs          Number of concurrent Large Objects jobs to run\n" \
 	"  --split-tables-larger-than    Same-table concurrency size threshold\n" \
+	"  --split-max-parts             Maximum number of jobs for Same-table concurrency \n" \
+	"  --estimate-table-sizes        Allow using estimates for relation sizes\n" \
 	"  --drop-if-exists              On the target database, clean-up from a previous run first\n" \
 	"  --roles                       Also copy roles found on source to target\n" \
 	"  --no-role-passwords           Do not dump passwords for roles\n" \
@@ -46,7 +48,9 @@
 	"  --skip-ext-comments           Skip restoring COMMENT ON EXTENSION\n" \
 	"  --skip-collations             Skip restoring collations\n" \
 	"  --skip-vacuum                 Skip running VACUUM ANALYZE\n" \
+	"  --skip-analyze                Skip running vacuumdb --analyze-only\n" \
 	"  --skip-db-properties          Skip copying ALTER DATABASE SET properties\n" \
+	"  --skip-split-by-ctid          Skip spliting tables by ctid\n" \
 	"  --requirements <filename>     List extensions requirements\n" \
 	"  --filters <filename>          Use the filters defined in <filename>\n" \
 	"  --fail-fast                   Abort early in case of error\n" \
@@ -61,6 +65,7 @@
 	"  --create-slot                 Create the replication slot\n" \
 	"  --origin                      Use this Postgres replication origin node name\n" \
 	"  --endpos                      Stop replaying changes when reaching this LSN\n" \
+	"  --use-copy-binary             Use the COPY BINARY format for COPY operations\n" \
 
 CommandLine clone_command =
 	make_command(
@@ -484,7 +489,7 @@ start_clone_process(CopyDataSpec *copySpecs, pid_t *pid)
 		case -1:
 		{
 			log_error("Failed to fork a subprocess to prefetch changes: %m");
-			return -1;
+			return false;
 		}
 
 		case 0:
@@ -578,9 +583,7 @@ cloneDB(CopyDataSpec *copySpecs)
 
 	log_info("STEP 2: dump the source database schema (pre/post data)");
 
-	if (!copydb_dump_source_schema(copySpecs,
-								   copySpecs->sourceSnapshot.snapshot,
-								   PG_DUMP_SECTION_SCHEMA))
+	if (!copydb_dump_source_schema(copySpecs, copySpecs->sourceSnapshot.snapshot))
 	{
 		/* errors have already been logged */
 		return false;
@@ -691,7 +694,7 @@ start_follow_process(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs,
 		case -1:
 		{
 			log_error("Failed to fork a subprocess to prefetch changes: %m");
-			return -1;
+			return false;
 		}
 
 		case 0:
