@@ -1,5 +1,5 @@
 /*
- * src/bin/pg_autoctl/cli_common.h
+ * src/bin/pgcopydb/cli_common.h
  *     Implementation of a CLI which lets you run individual keeper routines
  *     directly
  *
@@ -28,6 +28,22 @@ typedef struct SplitTableLargerThan
 } SplitTableLargerThan;
 
 
+typedef struct SentinelOptions
+{
+	/* pgcopydb stream sentinel get --flush-lsn */
+	bool startpos;
+	bool endpos;
+	bool apply;
+	bool writeLSN;
+	bool transformLSN;
+	bool flushLSN;
+	bool replayLSN;
+
+	/* pgcopydb stream sentinel set endpos --current */
+	bool currentLSN;
+} SentinelOptions;
+
+
 typedef struct CopyDBOptions
 {
 	char dir[MAXPGPATH];
@@ -39,6 +55,8 @@ typedef struct CopyDBOptions
 	int lObjectJobs;
 
 	SplitTableLargerThan splitTablesLargerThan;
+	int splitMaxParts;
+	bool estimateTableSizes;
 
 	RestoreOptions restoreOptions;
 
@@ -48,9 +66,12 @@ typedef struct CopyDBOptions
 	bool skipCommentOnExtension;
 	bool skipCollations;
 	bool skipVacuum;
+	bool skipAnalyze;
 	bool skipDBproperties;
+	bool skipCtidSplit;
 	bool noRolesPasswords;
 	bool failFast;
+	bool useCopyBinary;
 
 	bool restart;
 	bool resume;
@@ -65,9 +86,12 @@ typedef struct CopyDBOptions
 
 	bool follow;
 	bool createSlot;
-	bool currentpos;
+
+	/* pgcopydb stream sentinel get --flush-lsn and friends */
+	SentinelOptions sentinelOptions;
+
+	/* pgcopydb stream receive|transform|apply --endpos %X%X */
 	uint64_t endpos;
-	uint64_t startpos;
 
 	char filterFileName[MAXPGPATH];
 	char requirementsFileName[MAXPGPATH];
