@@ -96,8 +96,6 @@ static CommandLine list_tables_command =
 		"  --source            Postgres URI to the source database\n"
 		"  --filter <filename> Use the filters defined in <filename>\n"
 		"  --force             Force fetching catalogs again\n"
-		"  --cache             Cache table size in relation pgcopydb.pgcopydb_table_size\n"
-		"  --drop-cache        Drop relation pgcopydb.pgcopydb_table_size\n"
 		"  --list-skipped      List only tables that are setup to be skipped\n"
 		"  --without-pkey      List only tables that have no primary key\n",
 		cli_list_db_getopts,
@@ -112,7 +110,10 @@ static CommandLine list_table_parts_command =
 		"  --force                     Force fetching catalogs again\n"
 		"  --schema-name               Name of the schema where to find the table\n"
 		"  --table-name                Name of the target table\n"
-		"  --split-tables-larger-than  Size threshold to consider partitioning\n",
+		"  --split-tables-larger-than  Size threshold to consider partitioning\n"
+		"  --split-max-parts           Maximum number of jobs for Same-table concurrency \n" \
+		"  --skip-split-by-ctid        Skip the ctid split\n"
+		"  --estimate-table-sizes      Allow using estimates for relation sizes\n",
 		cli_list_db_getopts,
 		cli_list_table_parts);
 
@@ -209,6 +210,23 @@ cli_list_getenv(ListDBOptions *options)
 {
 	int errors = 0;
 
+	EnvParser parsers[] = {
+		{ PGCOPYDB_SPLIT_MAX_PARTS, ENV_TYPE_INT,
+		  &(options->splitMaxParts), 0, true, 1 },
+		{ PGCOPYDB_ESTIMATE_TABLE_SIZES, ENV_TYPE_BOOL,
+		  &(options->estimateTableSizes) },
+	};
+
+	int parserCount = sizeof(parsers) / sizeof(parsers[0]);
+
+	EnvParserArray parserArray = { .count = parserCount, .array = parsers };
+
+	if (!get_env_using_parsers(&parserArray))
+	{
+		/* errors have already been logged */
+		++errors;
+	}
+
 	if (!cli_copydb_getenv_source_pguri(&(options->connStrings.source_pguri)))
 	{
 		/* errors have already been logged */
@@ -245,10 +263,11 @@ cli_list_db_getopts(int argc, char **argv)
 		{ "list-skipped", no_argument, NULL, 'x' },
 		{ "without-pkey", no_argument, NULL, 'P' },
 		{ "split-tables-larger-than", required_argument, NULL, 'L' },
+		{ "split-max-parts", required_argument, NULL, 'u' },
 		{ "split-at", required_argument, NULL, 'L' },
+		{ "estimate-table-sizes", no_argument, NULL, 'm' },
+		{ "skip-split-by-ctid", no_argument, NULL, 'k' },
 		{ "force", no_argument, NULL, 'f' },
-		{ "cache", no_argument, NULL, 'c' },
-		{ "drop-cache", no_argument, NULL, 'C' },
 		{ "summary", no_argument, NULL, 'y' },
 		{ "available-versions", no_argument, NULL, 'a' },
 		{ "requirements", no_argument, NULL, 'r' },
@@ -275,7 +294,9 @@ cli_list_db_getopts(int argc, char **argv)
 		exit(EXIT_CODE_BAD_ARGS);
 	}
 
-	while ((c = getopt_long(argc, argv, "S:D:s:t:F:xPL:fcCyarJRIN:Vdzvqh",
+	const char *optstring = "S:D:s:t:F:xPL:u:k:mfyarJRIN:Vdzvqh";
+
+	while ((c = getopt_long(argc, argv, optstring,
 							long_options, &option_index)) != -1)
 	{
 		switch (c)
@@ -361,6 +382,26 @@ cli_list_db_getopts(int argc, char **argv)
 				break;
 			}
 
+			case 'u':
+			{
+				if (!stringToInt(optarg, &options.splitMaxParts) ||
+					options.splitMaxParts < 1)
+				{
+					log_fatal("Failed to parse --split-max-parts: \"%s\"",
+							  optarg);
+					++errors;
+				}
+				log_trace("--split-max-parts %d", options.splitMaxParts);
+				break;
+			}
+
+			case 'k':
+			{
+				options.skipCtidSplit = true;
+				log_trace("--skip-split-by-ctid");
+				break;
+			}
+
 			case 'f':
 			{
 				options.force = true;
@@ -368,29 +409,10 @@ cli_list_db_getopts(int argc, char **argv)
 				break;
 			}
 
-			case 'c':
+			case 'm':
 			{
-				if (options.dropCache)
-				{
-					log_fatal("Please choose either --cache or --drop-cache");
-					++errors;
-				}
-
-				options.cache = true;
-				log_trace("--cache");
-				break;
-			}
-
-			case 'C':
-			{
-				if (options.cache)
-				{
-					log_fatal("Please choose either --cache or --drop-cache");
-					++errors;
-				}
-
-				options.dropCache = true;
-				log_trace("--drop-cache");
+				options.estimateTableSizes = true;
+				log_trace("--estimate-table-sizes");
 				break;
 			}
 
@@ -1278,8 +1300,22 @@ cli_list_table_parts(int argc, char **argv)
 		exit(EXIT_CODE_QUIT);
 	}
 
-	if (IS_EMPTY_STRING_BUFFER(table->partKey))
+	if (IS_EMPTY_STRING_BUFFER(table->partKey) &&
+		streq(table->amname, "heap"))
 	{
+		if (copySpecs.skipCtidSplit)
+		{
+			log_info("Table %s is %s large "
+					 "which is larger than --split-tables-larger-than %s, "
+					 "does not have a unique column of type integer, "
+					 "and CTID split is disabled. "
+					 "Same table concurrency is not enabled",
+					 table->qname,
+					 table->bytesPretty,
+					 listDBoptions.splitTablesLargerThan.bytesPretty);
+			exit(EXIT_CODE_QUIT);
+		}
+
 		log_info("Table %s is %s large "
 				 "which is larger than --split-tables-larger-than %s, "
 				 "and does not have a unique column of type integer: "
@@ -1290,6 +1326,19 @@ cli_list_table_parts(int argc, char **argv)
 
 		strlcpy(table->partKey, "ctid", sizeof(table->partKey));
 	}
+	else if (IS_EMPTY_STRING_BUFFER(table->partKey))
+	{
+		log_info("Table %s is %s large "
+				 "which is larger than --split-tables-larger-than %s, "
+				 "does not have a unique column of type integer, "
+				 "and uses table access method \"%s\": "
+				 "same table concurrency is not enabled",
+				 table->qname,
+				 table->bytesPretty,
+				 listDBoptions.splitTablesLargerThan.bytesPretty,
+				 table->amname);
+		exit(EXIT_CODE_QUIT);
+	}
 
 	log_info("Table %s COPY will be split %d-ways",
 			 table->qname,
@@ -1996,6 +2045,9 @@ copydb_init_specs_from_listdboptions(CopyDataSpec *copySpecs,
 	strlcpy(options.dir, listDBoptions->dir, MAXPGPATH);
 	options.connStrings = listDBoptions->connStrings;
 	options.splitTablesLargerThan = listDBoptions->splitTablesLargerThan;
+	options.splitMaxParts = listDBoptions->splitMaxParts;
+	options.skipCtidSplit = listDBoptions->skipCtidSplit;
+	options.estimateTableSizes = listDBoptions->estimateTableSizes;
 
 	/* process the --resume --not-consistent --snapshot options now */
 	options.resume = listDBoptions->resume;
