@@ -29,17 +29,18 @@ typedef struct ListDBOptions
 	bool listSkipped;
 	bool noPKey;
 	bool force;
-	bool cache;
-	bool dropCache;
 	bool summary;
 	bool availableVersions;
 	bool requirements;
 	bool resume;
 	bool notConsistent;
+	bool skipCtidSplit;
 
 	char snapshot[BUFSIZE];
 
 	SplitTableLargerThan splitTablesLargerThan;
+	int splitMaxParts;
+	bool estimateTableSizes;
 } ListDBOptions;
 
 
