@@ -182,10 +182,6 @@ cli_restore_schema_getopts(int argc, char **argv)
 
 	optind = 0;
 
-	/* install default values */
-	options.indexJobs = DEFAULT_INDEX_JOBS;
-	options.restoreOptions.jobs = DEFAULT_RESTORE_JOBS;
-
 	/* read values from the environment */
 	if (!cli_copydb_getenv(&options))
 	{
@@ -553,6 +549,24 @@ cli_restore_roles(int argc, char **argv)
 }
 
 
+/*
+ * print_archive_toc_item_hook is an iterator callback function.
+ */
+static bool
+print_archive_toc_item_hook(void *context, ArchiveContentItem *item)
+{
+	fformat(stdout,
+			"%d; %u %u %s %s\n",
+			item->dumpId,
+			item->catalogOid,
+			item->objectOid,
+			item->description ? item->description : "",
+			item->restoreListName ? item->restoreListName : "");
+
+	return true;
+}
+
+
 /*
  * cli_restore_schema implements the command: pgcopydb restore parse-list
  */
@@ -565,29 +579,12 @@ cli_restore_schema_parse_list(int argc, char **argv)
 
 		log_info("Parsing Archive Content pre.list file: \"%s\"", filename);
 
-		ArchiveContentArray contents = { 0 };
-
-		if (!parse_archive_list(filename, &contents))
+		if (!archive_iter_toc(filename, NULL, print_archive_toc_item_hook))
 		{
 			/* errors have already been logged */
 			exit(EXIT_CODE_INTERNAL_ERROR);
 		}
 
-		log_notice("Read %d archive items in \"%s\"", contents.count, filename);
-
-		for (int i = 0; i < contents.count; i++)
-		{
-			ArchiveContentItem *item = &(contents.array[i]);
-
-			fformat(stdout,
-					"%d; %u %u %s %s\n",
-					item->dumpId,
-					item->catalogOid,
-					item->objectOid,
-					item->description ? item->description : "",
-					item->restoreListName ? item->restoreListName : "");
-		}
-
 		exit(EXIT_CODE_QUIT);
 	}
 
@@ -595,30 +592,9 @@ cli_restore_schema_parse_list(int argc, char **argv)
 
 	(void) cli_restore_prepare_specs(&copySpecs);
 
-	SourceFilters *filters = &(copySpecs.filters);
-
-	if (filters->type != SOURCE_FILTER_TYPE_NONE)
-	{
-		if (!copydb_prepare_snapshot(&copySpecs))
-		{
-			/* errors have already been logged */
-			exit(EXIT_CODE_INTERNAL_ERROR);
-		}
-
-		/* fetch schema information from source catalogs, including filtering */
-		if (!copydb_fetch_schema_and_prepare_specs(&copySpecs))
-		{
-			/* errors have already been logged */
-			(void) copydb_close_snapshot(&copySpecs);
-			exit(EXIT_CODE_TARGET);
-		}
-
-		(void) copydb_close_snapshot(&copySpecs);
-	}
-
 	log_info("Preparing the pg_restore --use-list for the pre-data "
 			 "archive file \"%s\" at: \"%s\"",
-			 copySpecs.dumpPaths.preFilename,
+			 copySpecs.dumpPaths.dumpFilename,
 			 copySpecs.dumpPaths.preListFilename);
 
 	if (!copydb_write_restore_list(&copySpecs, PG_DUMP_SECTION_PRE_DATA))
@@ -630,7 +606,7 @@ cli_restore_schema_parse_list(int argc, char **argv)
 
 	log_info("Preparing the pg_restore --use-list for the post-data "
 			 "archive file \"%s\" at: \"%s\"",
-			 copySpecs.dumpPaths.postFilename,
+			 copySpecs.dumpPaths.dumpFilename,
 			 copySpecs.dumpPaths.postListFilename);
 
 	if (!copydb_write_restore_list(&copySpecs, PG_DUMP_SECTION_POST_DATA))
