@@ -368,10 +368,6 @@ copydb_prepare_filepaths(CopyFilePaths *cfPaths,
 			"%s/slot",
 			cfPaths->cdc.dir);
 
-	sformat(cfPaths->cdc.tlihistfile, MAXPGPATH,
-			"%s/tli.history",
-			cfPaths->cdc.dir);
-
 	sformat(cfPaths->cdc.tlifile, MAXPGPATH,
 			"%s/tli",
 			cfPaths->cdc.dir);
@@ -423,8 +419,8 @@ copydb_prepare_dump_paths(CopyFilePaths *cfPaths, DumpPaths *dumpPaths)
 	sformat(dumpPaths->extnspFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "extnamespaces.dump");
 
-	sformat(dumpPaths->preFilename, MAXPGPATH, "%s/%s",
-			cfPaths->schemadir, "pre.dump");
+	sformat(dumpPaths->dumpFilename, MAXPGPATH, "%s/%s",
+			cfPaths->schemadir, "schema.dump");
 
 	sformat(dumpPaths->preListOutFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "pre-out.list");
@@ -432,9 +428,6 @@ copydb_prepare_dump_paths(CopyFilePaths *cfPaths, DumpPaths *dumpPaths)
 	sformat(dumpPaths->preListFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "pre-filtered.list");
 
-	sformat(dumpPaths->postFilename, MAXPGPATH, "%s/%s",
-			cfPaths->schemadir, "post.dump");
-
 	sformat(dumpPaths->postListOutFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "post-out.list");
 
@@ -514,9 +507,12 @@ copydb_init_specs(CopyDataSpec *specs,
 		.skipCommentOnExtension = options->skipCommentOnExtension,
 		.skipCollations = options->skipCollations,
 		.skipVacuum = options->skipVacuum,
+		.skipAnalyze = options->skipAnalyze,
 		.skipDBproperties = options->skipDBproperties,
+		.skipCtidSplit = options->skipCtidSplit,
 		.noRolesPasswords = options->noRolesPasswords,
 		.failFast = options->failFast,
+		.useCopyBinary = options->useCopyBinary,
 
 		.restart = options->restart,
 		.resume = options->resume,
@@ -535,9 +531,11 @@ copydb_init_specs(CopyDataSpec *specs,
 		.vacuumJobs = options->tableJobs,
 
 		.splitTablesLargerThan = options->splitTablesLargerThan,
+		.splitMaxParts = options->splitMaxParts,
+		.estimateTableSizes = options->estimateTableSizes,
 
-		.vacuumQueue = { 0 },
-		.indexQueue = { 0 },
+		.vacuumQueue = { NULL, -1 },
+		.indexQueue = { NULL, -1 },
 
 		.catalogs = { 0 }
 	};
@@ -574,17 +572,16 @@ copydb_init_specs(CopyDataSpec *specs,
 	strlcpy(filter->dbfile, specs->cfPaths.fdbfile, sizeof(filter->dbfile));
 	strlcpy(target->dbfile, specs->cfPaths.tdbfile, sizeof(target->dbfile));
 
-	if (specs->section == DATA_SECTION_ALL ||
-		specs->section == DATA_SECTION_TABLE_DATA)
+	bool shouldCreateVacuumQueue = (specs->section == DATA_SECTION_ALL ||
+									specs->section == DATA_SECTION_INDEXES ||
+									specs->section == DATA_SECTION_TABLE_DATA) &&
+								   !specs->skipVacuum;
+	if (shouldCreateVacuumQueue)
 	{
-		/* create the VACUUM process queue */
-		if (!specs->skipVacuum)
+		if (!queue_create(&(specs->vacuumQueue), "vacuum"))
 		{
-			if (!queue_create(&(specs->vacuumQueue), "vacuum"))
-			{
-				log_error("Failed to create the VACUUM process queue");
-				return false;
-			}
+			log_error("Failed to create the VACUUM process queue");
+			return false;
 		}
 	}
 
