@@ -104,6 +104,9 @@ typedef struct TransactionSnapshot
 	ConnectionType connectionType;
 
 	char snapshot[BUFSIZE];
+
+	/* indicator for read-only source db */
+	bool isReadOnly;
 } TransactionSnapshot;
 
 
@@ -224,8 +227,11 @@ typedef struct CopyDataSpec
 	bool skipCommentOnExtension;
 	bool skipCollations;
 	bool skipVacuum;
+	bool skipAnalyze;
 	bool skipDBproperties;
+	bool skipCtidSplit;
 	bool noRolesPasswords;
+	bool useCopyBinary;
 
 	bool restart;
 	bool resume;
@@ -243,6 +249,8 @@ typedef struct CopyDataSpec
 	int lObjectJobs;
 
 	SplitTableLargerThan splitTablesLargerThan;
+	int splitMaxParts;
+	bool estimateTableSizes;
 
 	Queue copyQueue;
 	Queue indexQueue;
@@ -258,19 +266,6 @@ typedef struct CopyDataSpec
 	Catalogs catalogs;
 } CopyDataSpec;
 
-
-/* specify section of a dump: pre-data, post-data, data, schema */
-typedef enum
-{
-	PG_DUMP_SECTION_ALL = 0,
-	PG_DUMP_SECTION_SCHEMA,
-	PG_DUMP_SECTION_PRE_DATA,
-	PG_DUMP_SECTION_POST_DATA,
-	PG_DUMP_SECTION_DATA,
-	PG_DUMP_SECTION_ROLES       /* pg_dumpall --roles-only */
-} PostgresDumpSection;
-
-
 extern GUC srcSettings95[];
 extern GUC srcSettings[];
 extern GUC dstSettings[];
@@ -387,9 +382,7 @@ bool copydb_prepare_create_constraint_command(CopyIndexSpec *indexSpecs);
 bool copydb_create_constraints(CopyDataSpec *spec, PGSQL *dst, SourceTable *table);
 
 /* dump_restore.c */
-bool copydb_dump_source_schema(CopyDataSpec *specs,
-							   const char *snapshot,
-							   PostgresDumpSection section);
+bool copydb_dump_source_schema(CopyDataSpec *specs, const char *snapshot);
 bool copydb_target_prepare_schema(CopyDataSpec *specs);
 bool copydb_copy_database_properties(CopyDataSpec *specs);
 bool copydb_target_drop_tables(CopyDataSpec *specs);
@@ -410,6 +403,8 @@ bool copydb_fetch_schema_and_prepare_specs(CopyDataSpec *specs);
 bool copydb_objectid_is_filtered_out(CopyDataSpec *specs,
 									 uint32_t oid,
 									 char *restoreListName);
+bool copydb_matview_refresh_is_filtered_out(CopyDataSpec *specs,
+											uint32_t oid);
 
 bool copydb_prepare_table_specs(CopyDataSpec *specs, PGSQL *pgsql);
 bool copydb_prepare_index_specs(CopyDataSpec *specs, PGSQL *pgsql);
@@ -534,6 +529,9 @@ bool summary_add_table(DatabaseCatalog *catalog,
 bool summary_finish_table(DatabaseCatalog *catalog,
 						  CopyTableDataSpec *tableSpecs);
 
+bool summary_update_table_copy_stats(DatabaseCatalog *catalog,
+									 CopyTableDataSpec *tableSpecs);
+
 bool summary_delete_table(DatabaseCatalog *catalog,
 						  CopyTableDataSpec *tableSpecs);
 
