@@ -18,7 +18,6 @@ typedef struct CDCPaths
 	char slotfile[MAXPGPATH];         /* /tmp/pgcopydb/cdc/slot */
 	char walsegsizefile[MAXPGPATH];   /* /tmp/pgcopydb/cdc/wal_segment_size */
 	char tlifile[MAXPGPATH];          /* /tmp/pgcopydb/cdc/tli */
-	char tlihistfile[MAXPGPATH];      /* /tmp/pgcopydb/cdc/tli.history */
 	char lsntrackingfile[MAXPGPATH];  /* /tmp/pgcopydb/cdc/lsn.json */
 } CDCPaths;
 
@@ -59,11 +58,11 @@ typedef struct DumpPaths
 	char rolesFilename[MAXPGPATH];   /* pg_dumpall --roles-only */
 	char extnspFilename[MAXPGPATH];  /* pg_dump --schema-only -n ... */
 
-	char preFilename[MAXPGPATH];     /* pg_dump --section=pre-data */
+	char dumpFilename[MAXPGPATH];     /* pg_dump --section=pre-data --section=post-data */
+
 	char preListOutFilename[MAXPGPATH]; /* pg_restore --list */
 	char preListFilename[MAXPGPATH]; /* pg_restore --use-list */
 
-	char postFilename[MAXPGPATH];     /* pg_dump --section=post-data */
 	char postListOutFilename[MAXPGPATH]; /* pg_restore --list */
 	char postListFilename[MAXPGPATH];    /* pg_restore --use-list */
 } DumpPaths;
