@@ -452,20 +452,12 @@ copydb_fetch_source_schema(CopyDataSpec *specs, PGSQL *src)
 {
 	DatabaseCatalog *sourceDB = &(specs->catalogs.source);
 
-	/* check if we're connected to a standby server, which we don't support */
-	bool pg_is_in_recovery = false;
-
-	if (!pgsql_is_in_recovery(src, &pg_is_in_recovery))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
-	if (pg_is_in_recovery)
+	if (specs->sourceSnapshot.isReadOnly && specs->filters.type !=
+		SOURCE_FILTER_TYPE_NONE)
 	{
 		log_fatal("Connected to a standby server where pg_is_in_recovery(): "
 				  "pgcopydb does not support operating on standby server "
-				  "at this point, as it needs to create temp tables");
+				  "when --filters are used, as it needs to create temp tables");
 		return false;
 	}
 
@@ -553,16 +545,6 @@ copydb_fetch_source_schema(CopyDataSpec *specs, PGSQL *src)
 		 specs->section == DATA_SECTION_TABLE_DATA_PARTS) &&
 		!sourceDB->sections[DATA_SECTION_TABLE_DATA].fetched)
 	{
-		/* copydb_fetch_filtered_oids() needs the table size table around */
-		if (!schema_prepare_pgcopydb_table_size(src,
-												&(specs->filters),
-												sourceDB))
-		{
-			/* errors have already been logged */
-			(void) semaphore_unlock(&(sourceDB->sema));
-			return false;
-		}
-
 		if (!copydb_prepare_table_specs(specs, src))
 		{
 			/* errors have already been logged */
@@ -611,7 +593,10 @@ copydb_fetch_source_schema(CopyDataSpec *specs, PGSQL *src)
 		}
 	}
 
-	/* now update --split-tables-larger-than and target pguri */
+	/*
+	 * now update target pguri, --split-tables-larger-than, and
+	 * --split-max-parts
+	 */
 	if (!catalog_update_setup(specs))
 	{
 		/* errors have already been logged */
@@ -667,16 +652,57 @@ copydb_prepare_table_specs(CopyDataSpec *specs, PGSQL *pgsql)
 
 	(void) catalog_start_timing(&timing);
 
+	/* if we're estimating table sizes, we need statistics */
+	if (specs->estimateTableSizes)
+	{
+		/* if --skip-analyze was used, we assume the user updated the statistics */
+		if (specs->skipAnalyze)
+		{
+			log_info("Skipping vacuumdb --analyze-only on source database "
+					 "before calculating table size estimates");
+		}
+		else
+		{
+			log_info("Running vacuumdb --analyze-only on source database before "
+					 "calculating table size estimates");
+
+			if (!pg_vacuumdb_analyze_only(&(specs->pgPaths),
+										  &(specs->connStrings),
+										  specs->tableJobs))
+			{
+				log_error("Failed to vacuum analyze source database, "
+						  "see above for details");
+				return false;
+			}
+		}
+	}
+
 	/*
 	 * Now get the list of the tables we want to COPY over.
+	 *
+	 * If we enabled estimates, we also update the table sizes in our internal
+	 * catalog using the relpages values.
 	 */
-	if (!schema_list_ordinary_tables(pgsql, filters, sourceDB))
+	if (!schema_list_ordinary_tables(pgsql, filters, specs->estimateTableSizes, sourceDB))
 	{
 		log_error("Failed to prepare table specs in our catalogs, "
 				  "see above for details");
 		return false;
 	}
 
+	/*
+	 * if we did not enable estimates, update table sizes in our internal
+	 * catalogue with exact values
+	 */
+	if (!specs->estimateTableSizes)
+	{
+		if (!schema_prepare_pgcopydb_table_size(pgsql, filters, sourceDB))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+	}
+
 	(void) catalog_stop_timing(&timing);
 
 	if (!catalog_register_section(sourceDB, &timing))
@@ -769,10 +795,37 @@ copydb_prepare_table_specs_hook(void *ctx, SourceTable *source)
 	 * When the Table Access Method used is not "heap" we don't know if the
 	 * CTID range scan is supported (see columnar storage extensions), so
 	 * we skip partitioning altogether in that case.
+	 *
+	 * Also, we cannot execute ANALYZE on a database that is in recovery mode,
+	 * so we skip partitioning in that case too.
 	 */
+
+	if (specs->sourceSnapshot.isReadOnly)
+	{
+		log_warn("Connected to a standby server where pg_is_in_recovery(): "
+				 "skipping partitioning for table %s",
+				 source->qname);
+
+		return true;
+	}
+
 	if (IS_EMPTY_STRING_BUFFER(source->partKey) &&
 		streq(source->amname, "heap"))
 	{
+		if (specs->skipCtidSplit)
+		{
+			log_info("Table %s is %s large "
+					 "which is larger than --split-tables-larger-than %s, "
+					 "does not have a unique column of type integer, "
+					 "and CTID split is disabled. "
+					 "Same table concurrency is not enabled",
+					 source->qname,
+					 source->bytesPretty,
+					 specs->splitTablesLargerThan.bytesPretty);
+
+			return true;
+		}
+
 		log_info("Table %s is %s large "
 				 "which is larger than --split-tables-larger-than %s, "
 				 "and does not have a unique column of type integer: "
@@ -787,17 +840,25 @@ copydb_prepare_table_specs_hook(void *ctx, SourceTable *source)
 		 * Make sure we have proper statistics (relpages) about the table
 		 * before compute the CTID ranges for the concurrent table scans.
 		 */
-		char sql[BUFSIZE] = { 0 };
-
-		sformat(sql, sizeof(sql), "ANALYZE %s", source->qname);
-
-		log_notice("%s", sql);
-
-		if (!pgsql_execute(context->pgsql, sql))
+		if (specs->estimateTableSizes)
 		{
-			log_error("Failed to refresh table %s statistics",
+			log_debug("Skipping running ANALYZE on table %s for CTID split",
 					  source->qname);
-			return false;
+		}
+		else
+		{
+			char sql[BUFSIZE] = { 0 };
+
+			sformat(sql, sizeof(sql), "ANALYZE %s", source->qname);
+
+			log_notice("%s", sql);
+
+			if (!pgsql_execute(context->pgsql, sql))
+			{
+				log_error("Failed to refresh table %s statistics",
+						  source->qname);
+				return false;
+			}
 		}
 
 		/* fetch the relpages for the table after ANALYZE */
@@ -832,7 +893,8 @@ copydb_prepare_table_specs_hook(void *ctx, SourceTable *source)
 	if (!schema_list_partitions(context->pgsql,
 								sourceDB,
 								source,
-								specs->splitTablesLargerThan.bytes))
+								specs->splitTablesLargerThan.bytes,
+								specs->splitMaxParts))
 	{
 		/* errors have already been logged */
 		return false;
@@ -911,6 +973,44 @@ copydb_prepare_index_specs(CopyDataSpec *specs, PGSQL *pgsql)
 }
 
 
+/*
+ * copydb_matview_refresh_is_filtered_out returns true when the
+ * given oid belongs to a materialized view object that's
+ * data(refresh) has been filtered out by the filtering setup using
+ * [exclude-table-data].
+ */
+bool
+copydb_matview_refresh_is_filtered_out(CopyDataSpec *specs, uint32_t oid)
+{
+	/*
+	 * We need to check the s_matview table exists on the source catalog
+	 * to find out if the materialized view refresh has been filtered out.
+	 *
+	 * The filtering of materialized view as a whole handled by the
+	 * existing filtering setup(i.e. copydb_objectid_is_filtered_out).
+	 */
+	DatabaseCatalog *sourceDB = &(specs->catalogs.source);
+
+	CatalogMatView result = { 0 };
+
+	if (oid != 0)
+	{
+		if (!catalog_lookup_s_matview_by_oid(sourceDB, &result, oid))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+
+		if (result.oid != 0 && result.excludeData)
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
 /*
  * copydb_objectid_is_filtered_out returns true when the given oid belongs to a
  * database object that's been filtered out by the filtering setup.
@@ -1125,7 +1225,8 @@ copydb_fetch_filtered_oids(CopyDataSpec *specs, PGSQL *pgsql)
 
 		(void) catalog_start_timing(&timing);
 
-		if (!schema_list_ordinary_tables(pgsql, filters, filtersDB))
+		if (!schema_list_ordinary_tables(pgsql, filters, specs->estimateTableSizes,
+										 filtersDB))
 		{
 			/* errors have already been logged */
 			filters->type = type;
