@@ -26,9 +26,13 @@
 
 
 static bool copydb_append_table_hook(void *context, SourceTable *table);
+
 static bool copydb_copy_database_properties_hook(void *ctx,
 												 SourceProperty *property);
 
+static bool copydb_write_restore_list_hook(void *ctx,
+										   ArchiveContentItem *item);
+
 
 /*
  * copydb_objectid_has_been_processed_already returns true when the given
@@ -87,13 +91,12 @@ copydb_objectid_has_been_processed_already(CopyDataSpec *specs,
 
 
 /*
- * copydb_dump_source_schema uses pg_dump -Fc --schema --section=pre-data or
- * --section=post-data to dump the source database schema to files.
+ * copydb_dump_source_schema uses pg_dump -Fc --schema --schema-only
+ * to dump the source database schema to file.
  */
 bool
 copydb_dump_source_schema(CopyDataSpec *specs,
-						  const char *snapshot,
-						  PostgresDumpSection section)
+						  const char *snapshot)
 {
 	DatabaseCatalog *sourceDB = &(specs->catalogs.source);
 
@@ -110,50 +113,21 @@ copydb_dump_source_schema(CopyDataSpec *specs,
 		return false;
 	}
 
-	if (section == PG_DUMP_SECTION_SCHEMA ||
-		section == PG_DUMP_SECTION_PRE_DATA ||
-		section == PG_DUMP_SECTION_ALL)
+	if (file_exists(specs->dumpPaths.dumpFilename))
 	{
-		if (file_exists(specs->dumpPaths.preFilename))
-		{
-			log_info("Skipping pg_dump --section=pre-data, "
-					 "as \"%s\" already exists",
-					 specs->dumpPaths.preFilename);
-		}
-		else if (!pg_dump_db(&(specs->pgPaths),
-							 &(specs->connStrings),
-							 snapshot,
-							 "pre-data",
-							 &(specs->filters),
-							 &(specs->catalogs.filter),
-							 specs->dumpPaths.preFilename))
-		{
-			/* errors have already been logged */
-			return false;
-		}
+		log_info("Skipping source schema dump, "
+				 "as \"%s\" already exists",
+				 specs->dumpPaths.dumpFilename);
 	}
-
-	if (section == PG_DUMP_SECTION_SCHEMA ||
-		section == PG_DUMP_SECTION_POST_DATA ||
-		section == PG_DUMP_SECTION_ALL)
+	else if (!pg_dump_db(&(specs->pgPaths),
+						 &(specs->connStrings),
+						 snapshot,
+						 &(specs->filters),
+						 &(specs->catalogs.filter),
+						 specs->dumpPaths.dumpFilename))
 	{
-		if (file_exists(specs->dumpPaths.postFilename))
-		{
-			log_info("Skipping pg_dump --section=post-data, "
-					 "as \"%s\" already exists",
-					 specs->dumpPaths.postFilename);
-		}
-		else if (!pg_dump_db(&(specs->pgPaths),
-							 &(specs->connStrings),
-							 snapshot,
-							 "post-data",
-							 &(specs->filters),
-							 &(specs->catalogs.filter),
-							 specs->dumpPaths.postFilename))
-		{
-			/* errors have already been logged */
-			return false;
-		}
+		/* errors have already been logged */
+		return false;
 	}
 
 	if (!summary_stop_timing(sourceDB, TIMING_SECTION_DUMP_SCHEMA))
@@ -167,7 +141,7 @@ copydb_dump_source_schema(CopyDataSpec *specs,
 
 
 /*
- * copydb_target_prepare_schema restores the pre.dump file into the target
+ * copydb_target_prepare_schema restores the schema.dump file into the target
  * database.
  */
 bool
@@ -175,9 +149,9 @@ copydb_target_prepare_schema(CopyDataSpec *specs)
 {
 	DatabaseCatalog *sourceDB = &(specs->catalogs.source);
 
-	if (!file_exists(specs->dumpPaths.preFilename))
+	if (!file_exists(specs->dumpPaths.dumpFilename))
 	{
-		log_fatal("File \"%s\" does not exists", specs->dumpPaths.preFilename);
+		log_fatal("File \"%s\" does not exists", specs->dumpPaths.dumpFilename);
 		return false;
 	}
 
@@ -231,10 +205,11 @@ copydb_target_prepare_schema(CopyDataSpec *specs)
 		}
 	}
 
+	specs->restoreOptions.section = PG_RESTORE_SECTION_PRE_DATA;
 	if (!pg_restore_db(&(specs->pgPaths),
 					   &(specs->connStrings),
 					   &(specs->filters),
-					   specs->dumpPaths.preFilename,
+					   specs->dumpPaths.dumpFilename,
 					   specs->dumpPaths.preListFilename,
 					   specs->restoreOptions))
 	{
@@ -548,9 +523,9 @@ copydb_target_finalize_schema(CopyDataSpec *specs)
 {
 	DatabaseCatalog *sourceDB = &(specs->catalogs.source);
 
-	if (!file_exists(specs->dumpPaths.postFilename))
+	if (!file_exists(specs->dumpPaths.dumpFilename))
 	{
-		log_fatal("File \"%s\" does not exists", specs->dumpPaths.postFilename);
+		log_fatal("File \"%s\" does not exists", specs->dumpPaths.dumpFilename);
 		return false;
 	}
 
@@ -574,10 +549,11 @@ copydb_target_finalize_schema(CopyDataSpec *specs)
 		return false;
 	}
 
+	specs->restoreOptions.section = PG_RESTORE_SECTION_POST_DATA;
 	if (!pg_restore_db(&(specs->pgPaths),
 					   &(specs->connStrings),
 					   &(specs->filters),
-					   specs->dumpPaths.postFilename,
+					   specs->dumpPaths.dumpFilename,
 					   specs->dumpPaths.postListFilename,
 					   specs->restoreOptions))
 	{
@@ -595,6 +571,12 @@ copydb_target_finalize_schema(CopyDataSpec *specs)
 }
 
 
+typedef struct RestoreListContext
+{
+	CopyDataSpec *specs;
+	FILE *outStream;
+} RestoreListContext;
+
 /*
  * copydb_write_restore_list fetches the pg_restore --list output, parses it,
  * and then writes it again to file and applies the filtering to the archive
@@ -611,7 +593,7 @@ copydb_write_restore_list(CopyDataSpec *specs, PostgresDumpSection section)
 	{
 		case PG_DUMP_SECTION_PRE_DATA:
 		{
-			dumpFilename = specs->dumpPaths.preFilename;
+			dumpFilename = specs->dumpPaths.dumpFilename;
 			listFilename = specs->dumpPaths.preListFilename;
 			listOutFilename = specs->dumpPaths.preListOutFilename;
 			break;
@@ -619,7 +601,7 @@ copydb_write_restore_list(CopyDataSpec *specs, PostgresDumpSection section)
 
 		case PG_DUMP_SECTION_POST_DATA:
 		{
-			dumpFilename = specs->dumpPaths.postFilename;
+			dumpFilename = specs->dumpPaths.dumpFilename;
 			listFilename = specs->dumpPaths.postListFilename;
 			listOutFilename = specs->dumpPaths.postListOutFilename;
 			break;
@@ -635,166 +617,207 @@ copydb_write_restore_list(CopyDataSpec *specs, PostgresDumpSection section)
 	}
 
 	/*
-	 * The pre.dump archive file contains all the objects to create in the
+	 * The schema.dump archive file contains all the objects to create in the
 	 * target database. We want to filter out the schemas and tables excluded
 	 * from the filtering setup.
 	 *
-	 * The post.dump archive file contains all the objects to create once the
+	 * This archive file also contains all the objects to create once the
 	 * table data has been copied over. It contains in particular the
 	 * constraints and indexes that we have already built concurrently in the
 	 * previous step, so we want to filter those out.
 	 *
 	 * Here's how to filter out some objects with pg_restore:
 	 *
-	 *   1. pg_restore -f post.list --list post.dump
+	 *   1. pg_restore -f post.list --list schema.dump
 	 *   2. edit post.list to comment out lines and save as filtered.list
-	 *   3. pg_restore --use-list filtered.list post.dump
+	 *   3. pg_restore --section post-data --use-list filtered.list schema.dump
 	 */
-	ArchiveContentArray contents = { 0 };
-
 	if (!pg_restore_list(&(specs->pgPaths),
 						 dumpFilename,
-						 listOutFilename,
-						 &contents))
+						 listOutFilename))
 	{
 		/* errors have already been logged */
 		return false;
 	}
 
 	/* edit our post.list file now */
-	PQExpBuffer listContents = createPQExpBuffer();
+	FILE *out = fopen_with_umask(listFilename, "ab", FOPEN_FLAGS_A, 0644);
 
-	if (listContents == NULL)
-	{
-		log_error(ALLOCATION_FAILED_ERROR);
-		destroyPQExpBuffer(listContents);
-		return false;
-	}
-
-	/* for each object in the list, comment when we already processed it */
-	for (int i = 0; i < contents.count; i++)
-	{
-		ArchiveContentItem *item = &(contents.array[i]);
-		uint32_t oid = item->objectOid;
-		uint32_t catOid = item->catalogOid;
-		char *name = item->restoreListName;
-
-		bool skip = false;
-
-		/*
-		 * Always skip DATABASE objets as pgcopydb does not create the target
-		 * database.
-		 */
-		if (item->desc == ARCHIVE_TAG_DATABASE)
-		{
-			skip = true;
-			log_notice("Skipping DATABASE \"%s\"", name);
-		}
-
-		/*
-		 * Skip COMMENT ON EXTENSION when either of the option
-		 * --skip-extensions or --skip-ext-comment has been used.
-		 */
-		if ((specs->skipExtensions ||
-			 specs->skipCommentOnExtension) &&
-			item->isCompositeTag &&
-			item->tagKind == ARCHIVE_TAG_KIND_COMMENT &&
-			item->tagType == ARCHIVE_TAG_TYPE_EXTENSION)
-		{
-			skip = true;
-			log_notice("Skipping COMMENT ON EXTENSION \"%s\"", name);
-		}
-
-		if (!skip && catOid == PG_NAMESPACE_OID)
-		{
-			bool exists = false;
-
-			if (!copydb_schema_already_exists(specs, oid, &exists))
-			{
-				log_error("Failed to check if restore name \"%s\" "
-						  "already exists",
-						  name);
-				destroyPQExpBuffer(listContents);
-				return false;
-			}
-
-			if (exists)
-			{
-				skip = true;
-
-				log_notice("Skipping already existing dumpId %d: %s %u %s",
-						   contents.array[i].dumpId,
-						   contents.array[i].description,
-						   contents.array[i].objectOid,
-						   contents.array[i].restoreListName);
-			}
-		}
-
-		if (!skip && copydb_objectid_has_been_processed_already(specs, item))
-		{
-			skip = true;
-
-			log_notice("Skipping already processed dumpId %d: %s %u %s",
-					   contents.array[i].dumpId,
-					   contents.array[i].description,
-					   contents.array[i].objectOid,
-					   contents.array[i].restoreListName);
-		}
-
-		/*
-		 * For SEQUENCE catalog entries, we want to limit the scope of the hash
-		 * table search to the OID, and bypass searching by restore name. We
-		 * only use the restore name for the SEQUENCE OWNED BY statements.
-		 *
-		 * This also allows complex filtering of sequences that are owned by
-		 * table a and used as a default value in table b, where table a has
-		 * been filtered-out from pgcopydb scope of operations, but not table
-		 * b.
-		 */
-		if (item->desc == ARCHIVE_TAG_SEQUENCE)
-		{
-			name = NULL;
-		}
-
-		if (!skip && copydb_objectid_is_filtered_out(specs, oid, name))
-		{
-			skip = true;
-
-			log_notice("Skipping filtered-out dumpId %d: %s %u %u %s",
-					   contents.array[i].dumpId,
-					   contents.array[i].description,
-					   contents.array[i].catalogOid,
-					   contents.array[i].objectOid,
-					   contents.array[i].restoreListName);
-		}
-
-		appendPQExpBuffer(listContents, "%s%d; %u %u %s %s\n",
-						  skip ? ";" : "",
-						  contents.array[i].dumpId,
-						  contents.array[i].catalogOid,
-						  contents.array[i].objectOid,
-						  contents.array[i].description,
-						  contents.array[i].restoreListName);
-	}
-
-	/* memory allocation could have failed while building string */
-	if (PQExpBufferBroken(listContents))
-	{
-		log_error("Failed to create pg_restore list file: out of memory");
-		destroyPQExpBuffer(listContents);
-		return false;
-	}
-
-	log_notice("Write filtered pg_restore list file at \"%s\"", listFilename);
-
-	if (!write_file(listContents->data, listContents->len, listFilename))
+	if (out == NULL)
 	{
 		/* errors have already been logged */
-		destroyPQExpBuffer(listContents);
 		return false;
 	}
 
-	destroyPQExpBuffer(listContents);
+	RestoreListContext context = {
+		.specs = specs,
+		.outStream = out
+	};
+
+	if (!archive_iter_toc(listOutFilename,
+						  &context,
+						  copydb_write_restore_list_hook))
+	{
+		log_error("Failed to prepare the pg_restore list file, "
+				  "see above for details");
+		return false;
+	}
+
+	if (fclose(out) == EOF)
+	{
+		log_error("Failed to write file \"%s\"", listFilename);
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * copydb_write_restore_list_hook is an iterator callback function.
+ */
+static bool
+copydb_write_restore_list_hook(void *ctx, ArchiveContentItem *item)
+{
+	RestoreListContext *context = (RestoreListContext *) ctx;
+	CopyDataSpec *specs = context->specs;
+
+	uint32_t oid = item->objectOid;
+	uint32_t catOid = item->catalogOid;
+	char *name = item->restoreListName;
+
+	bool skip = false;
+
+	/*
+	 * Always skip DATABASE objets as pgcopydb does not create the target
+	 * database.
+	 */
+	if (item->desc == ARCHIVE_TAG_DATABASE)
+	{
+		skip = true;
+		log_notice("Skipping DATABASE \"%s\"", name);
+	}
+
+	/*
+	 * Skip COMMENT ON EXTENSION when either of the option
+	 * --skip-extensions or --skip-ext-comment has been used.
+	 */
+	if ((specs->skipExtensions ||
+		 specs->skipCommentOnExtension) &&
+		item->isCompositeTag &&
+		item->tagKind == ARCHIVE_TAG_KIND_COMMENT &&
+		item->tagType == ARCHIVE_TAG_TYPE_EXTENSION)
+	{
+		skip = true;
+		log_notice("Skipping COMMENT ON EXTENSION \"%s\"", name);
+	}
+
+	if (!skip && catOid == PG_NAMESPACE_OID)
+	{
+		bool exists = false;
+
+		if (!copydb_schema_already_exists(specs, oid, &exists))
+		{
+			log_error("Failed to check if restore name \"%s\" "
+					  "already exists",
+					  name);
+			return false;
+		}
+
+		if (exists)
+		{
+			skip = true;
+
+			log_notice("Skipping already existing dumpId %d: %s %u %s",
+					   item->dumpId,
+					   item->description,
+					   item->objectOid,
+					   item->restoreListName);
+		}
+	}
+
+	if (!skip && copydb_objectid_has_been_processed_already(specs, item))
+	{
+		skip = true;
+
+		log_notice("Skipping already processed dumpId %d: %s %u %s",
+				   item->dumpId,
+				   item->description,
+				   item->objectOid,
+				   item->restoreListName);
+	}
+
+	/*
+	 * For SEQUENCE catalog entries, we want to limit the scope of the hash
+	 * table search to the OID, and bypass searching by restore name. We
+	 * only use the restore name for the SEQUENCE OWNED BY statements.
+	 *
+	 * This also allows complex filtering of sequences that are owned by
+	 * table a and used as a default value in table b, where table a has
+	 * been filtered-out from pgcopydb scope of operations, but not table
+	 * b.
+	 */
+	if (item->desc == ARCHIVE_TAG_SEQUENCE)
+	{
+		name = NULL;
+	}
+
+	/*
+	 * There could be a case where the materalized view is included in the
+	 * dump, but the refresh is filtered out using [exclude-table-data].
+	 * In this case, we want to skip the refresh.
+	 */
+	if (!skip &&
+		item->desc == ARCHIVE_TAG_REFRESH_MATERIALIZED_VIEW &&
+		copydb_matview_refresh_is_filtered_out(specs, oid))
+	{
+		skip = true;
+
+		log_notice("Skipping materialized view refresh dumpId %d: %s %u %s",
+				   item->dumpId,
+				   item->description,
+				   item->objectOid,
+				   item->restoreListName);
+	}
+
+	if (!skip && copydb_objectid_is_filtered_out(specs, oid, name))
+	{
+		skip = true;
+
+		log_notice("Skipping filtered-out dumpId %d: %s %u %u %s",
+				   item->dumpId,
+				   item->description,
+				   item->catalogOid,
+				   item->objectOid,
+				   item->restoreListName);
+	}
+
+	PQExpBuffer buf = createPQExpBuffer();
+
+	printfPQExpBuffer(buf, "%s%d; %u %u %s %s\n",
+					  skip ? ";" : "",
+					  item->dumpId,
+					  item->catalogOid,
+					  item->objectOid,
+					  item->description,
+					  item->restoreListName);
+
+	/* memory allocation could have failed while building string */
+	if (PQExpBufferBroken(buf))
+	{
+		log_error("Failed to create pg_restore list file: out of memory");
+		destroyPQExpBuffer(buf);
+		return false;
+	}
+
+	if (!write_to_stream(context->outStream, buf->data, buf->len))
+	{
+		log_error("Failed to write pg_restore list file, "
+				  "see above for details");
+		return false;
+	}
+
+	destroyPQExpBuffer(buf);
 
 	return true;
 }
