@@ -184,7 +184,7 @@ follow_init_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel)
 
 /*
  * follow_get_sentinel refreshes the given CopyDBSentinel with the current
- * values from the pgcopydb.sentinel table on the source database.
+ * values from the pgcopydb.sentinel table.
  */
 bool
 follow_get_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel, bool verbose)
@@ -419,12 +419,11 @@ follow_prepare_mode_switch(StreamSpecs *streamSpecs,
 
 	if (streamSpecs->system.timeline == 0)
 	{
-		if (!stream_read_context(&(streamSpecs->paths),
-								 &(streamSpecs->system),
-								 &(streamSpecs->WalSegSz)))
+		if (!stream_read_context(streamSpecs))
 		{
 			log_error("Failed to read the streaming context information "
-					  "from the source database, see above for details");
+					  "from the source database and internal catalogs, "
+					  "see above for details");
 			return false;
 		}
 	}
