@@ -208,12 +208,11 @@ stream_apply_setup(StreamSpecs *specs, StreamApplyContext *context)
 
 	if (specs->system.timeline == 0)
 	{
-		if (!stream_read_context(&(specs->paths),
-								 &(specs->system),
-								 &(specs->WalSegSz)))
+		if (!stream_read_context(specs))
 		{
 			log_error("Failed to read the streaming context information "
-					  "from the source database, see above for details");
+					  "from the source database and internal catalogs, "
+					  "see above for details");
 			return false;
 		}
 	}
@@ -279,9 +278,9 @@ stream_apply_cleanup(StreamApplyContext *context)
 
 
 /*
- * stream_apply_wait_for_sentinel fetches the current pgcopydb sentinel values
- * on the source database: the catchup processing only gets to start when the
- * sentinel "apply" column has been set to true.
+ * stream_apply_wait_for_sentinel fetches the current pgcopydb sentinel values:
+ * the catchup processing only gets to start when the sentinel "apply" column
+ * has been set to true.
  */
 bool
 stream_apply_wait_for_sentinel(StreamSpecs *specs, StreamApplyContext *context)
