@@ -54,13 +54,27 @@ static bool canCoalesceLogicalTransactionStatement(LogicalTransaction *txn,
 static bool coalesceLogicalTransactionStatement(LogicalTransaction *txn,
 												LogicalTransactionStatement *new);
 
+static bool markGeneratedColumnsFromTransaction(GeneratedColumnsCache *cache,
+												LogicalTransaction *txn);
+static bool markGeneratedColumnsFromStatement(GeneratedColumnsCache *cache,
+											  LogicalTransactionStatement *stmt);
+
+static bool prepareGeneratedColumnsCache_hook(void *ctx, SourceTable *table);
+
+static bool prepareGeneratedColumnsCache(StreamSpecs *specs);
+
+static bool isGeneratedColumn(GeneratedColumnSet *columns, const char *attname);
+
+static GeneratedColumnSet * lookupGeneratedColumnsForTable(GeneratedColumnsCache *cache,
+														   const char *nspname,
+														   const char *relname);
+
 /*
- * stream_transform_context_init_pgsql initializes StreamContext's
- * transformPGSQL and opens a connection to the target. This is required to use
- * PQescapeIdentifier API of libpq when escaping identifiers
+ * stream_transform_context_init initializes StreamContext for the transform
+ * operation.
  */
 bool
-stream_transform_context_init_pgsql(StreamSpecs *specs)
+stream_transform_context_init(StreamSpecs *specs)
 {
 	StreamContext *privateContext = &(specs->private);
 
@@ -81,6 +95,16 @@ stream_transform_context_init_pgsql(StreamSpecs *specs)
 		return false;
 	}
 
+	/*
+	 * Prepare the generated columns cache, which helps to skip the generated
+	 * columns in the SQL output.
+	 */
+	if (!prepareGeneratedColumnsCache(specs))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
 	return true;
 }
 
@@ -93,7 +117,7 @@ stream_transform_context_init_pgsql(StreamSpecs *specs)
 bool
 stream_transform_stream(StreamSpecs *specs)
 {
-	if (!stream_transform_context_init_pgsql(specs))
+	if (!stream_transform_context_init(specs))
 	{
 		/* errors have already been logged */
 		return false;
@@ -259,12 +283,11 @@ stream_transform_resume(StreamSpecs *specs)
 	/* we need timeline and wal_segment_size to compute WAL filenames */
 	if (specs->system.timeline == 0)
 	{
-		if (!stream_read_context(&(specs->paths),
-								 &(specs->system),
-								 &(specs->WalSegSz)))
+		if (!stream_read_context(specs))
 		{
 			log_error("Failed to read the streaming context information "
-					  "from the source database, see above for details");
+					  "from the source database and internal catalogs, "
+					  "see above for details");
 			return false;
 		}
 	}
@@ -439,6 +462,24 @@ stream_transform_write_message(StreamContext *privateContext,
 		txn->commit = true;
 	}
 
+	/*
+	 * Before serializing the transaction to disk or stdout, we need to find
+	 * the generated columns from the transactionn and mark them as such.
+	 *
+	 * It will help to set the value of the generated columns to DEFAULT in the
+	 * SQL output.
+	 */
+	GeneratedColumnsCache *cache = privateContext->generatedColumnsCache;
+
+	if (currentMsg->isTransaction && cache != NULL)
+	{
+		if (!markGeneratedColumnsFromTransaction(cache, txn))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+	}
+
 	/* now write the transaction out */
 	if (privateContext->out != NULL)
 	{
@@ -627,9 +668,9 @@ stream_transform_worker(StreamSpecs *specs)
 	 * The timeline and wal segment size are determined when connecting to the
 	 * source database, and stored to local files at that time. When the Stream
 	 * Transform Worker process is created, that information is read from our
-	 * local files.
+	 * local files and internal catalogs.
 	 */
-	if (!stream_read_context(&(specs->paths), &(specs->system), &(specs->WalSegSz)))
+	if (!stream_read_context(specs))
 	{
 		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
 		{
@@ -638,7 +679,8 @@ stream_transform_worker(StreamSpecs *specs)
 		}
 
 		log_error("Failed to read the streaming context information "
-				  "from the source database, see above for details");
+				  "from the source database and internal catalogs, "
+				  "see above for details");
 		return false;
 	}
 
@@ -662,7 +704,7 @@ stream_transform_from_queue(StreamSpecs *specs)
 		return false;
 	}
 
-	if (!stream_transform_context_init_pgsql(specs))
+	if (!stream_transform_context_init(specs))
 	{
 		/* errors have already been logged */
 		return false;
@@ -1455,7 +1497,7 @@ canCoalesceLogicalTransactionStatement(LogicalTransaction *txn,
 	LogicalMessageTuple *newInsertColumns = newInsert->new.array;
 
 	/* Last and current statements must have same number of columns */
-	if (lastInsertColumns->cols != newInsertColumns->cols)
+	if (lastInsertColumns->attributes.count != newInsertColumns->attributes.count)
 	{
 		return false;
 	}
@@ -1471,7 +1513,7 @@ canCoalesceLogicalTransactionStatement(LogicalTransaction *txn,
 	 * TODO: This parameter limit check is not applicable for COPY operations.
 	 * It should be removed once we switch to using COPY.
 	 */
-	if (((lastValuesArray->count + 1) * lastInsertColumns->cols) >
+	if (((lastValuesArray->count + 1) * lastInsertColumns->attributes.count) >
 		PQ_QUERY_PARAM_MAX_LIMIT)
 	{
 		return false;
@@ -1479,9 +1521,11 @@ canCoalesceLogicalTransactionStatement(LogicalTransaction *txn,
 
 
 	/* Last and current statements cols must have same name and order */
-	for (int i = 0; i < lastInsertColumns->cols; i++)
+	for (int i = 0; i < lastInsertColumns->attributes.count; i++)
 	{
-		if (!streq(lastInsertColumns->columns[i], newInsertColumns->columns[i]))
+		LogicalMessageAttribute *lastAttr = &(lastInsertColumns->attributes.array[i]);
+		LogicalMessageAttribute *newAttr = &(newInsertColumns->attributes.array[i]);
+		if (!streq(lastAttr->attname, newAttr->attname))
 		{
 			return false;
 		}
@@ -1564,23 +1608,25 @@ streamLogicalTransactionAppendStatement(LogicalTransaction *txn,
 bool
 AllocateLogicalMessageTuple(LogicalMessageTuple *tuple, int count)
 {
-	tuple->cols = count;
+	tuple->attributes.count = count;
 
 	if (count == 0)
 	{
-		tuple->columns = NULL;
-
 		LogicalMessageValuesArray *valuesArray = &(tuple->values);
 		valuesArray->count = 0;
 		valuesArray->capacity = 0;
 		valuesArray->array = NULL;
 
+		tuple->attributes.array = NULL;
+
 		return true;
 	}
 
-	tuple->columns = (char **) calloc(count, sizeof(char *));
+	tuple->attributes.array = (LogicalMessageAttribute *) calloc(count,
+																 sizeof(
+																	 LogicalMessageAttribute));
 
-	if (tuple->columns == NULL)
+	if (tuple->attributes.array == NULL)
 	{
 		log_error(ALLOCATION_FAILED_ERROR);
 		return false;
@@ -2032,11 +2078,17 @@ stream_write_insert(FILE *out, LogicalMessageInsert *insert)
 		/* loop over column names and add them to the out stream */
 		appendPQExpBuffer(buf, "%s", "(");
 
-		for (int c = 0; c < stmt->cols; c++)
+		LogicalMessageAttribute *attr = &(stmt->attributes.array[0]);
+
+		for (int c = 0; c < stmt->attributes.count; c++)
 		{
-			appendPQExpBuffer(buf, "%s%s",
-							  c > 0 ? ", " : "",
-							  stmt->columns[c]);
+			/* skip generated columns */
+			if (!attr[c].isgenerated)
+			{
+				appendPQExpBuffer(buf, "%s%s",
+								  c > 0 ? ", " : "",
+								  attr[c].attname);
+			}
 		}
 
 		appendPQExpBuffer(buf, "%s", ")");
@@ -2071,15 +2123,26 @@ stream_write_insert(FILE *out, LogicalMessageInsert *insert)
 			{
 				LogicalMessageValue *value = &(values->array[v]);
 
-				appendPQExpBuffer(buf, "%s$%d",
-								  v > 0 ? ", " : "",
-								  ++pos);
-
-				if (!stream_add_value_in_json_array(value, jsArray))
+				/*
+				 * Instead of skipping the generated column, we could have
+				 * set the value to DEFAULT. But, PG13 doesn't allow multi
+				 * value INSERT with DEFAULT for generated columns.
+				 *
+				 * TODO: Once we stop supporting PG13, set the value to DEFAULT
+				 * for generated columns similar to UPDATE.
+				 */
+				if (!attr[v].isgenerated)
 				{
-					/* errors have already been logged */
-					destroyPQExpBuffer(buf);
-					return false;
+					appendPQExpBuffer(buf, "%s$%d",
+									  v > 0 ? ", " : "",
+									  ++pos);
+
+					if (!stream_add_value_in_json_array(value, jsArray))
+					{
+						/* errors have already been logged */
+						destroyPQExpBuffer(buf);
+						return false;
+					}
 				}
 			}
 
@@ -2168,15 +2231,15 @@ stream_write_update(FILE *out, LogicalMessageUpdate *update)
 			/* now loop over column values for this VALUES row */
 			for (int v = 0; v < values->cols; v++)
 			{
-				const char *colname = new->columns[v];
+				LogicalMessageAttribute *attr = &(new->attributes.array[v]);
 				LogicalMessageValue *value = &(values->array[v]);
 
-				if (new->cols <= v)
+				if (new->attributes.count <= v)
 				{
 					log_error("Failed to write UPDATE statement with more "
 							  "VALUES (%d) than COLUMNS (%d)",
 							  values->cols,
-							  new->cols);
+							  new->attributes.count);
 					destroyPQExpBuffer(buf);
 					return false;
 				}
@@ -2188,13 +2251,14 @@ stream_write_update(FILE *out, LogicalMessageUpdate *update)
 				 */
 				bool skip = false;
 
-				for (int oc = 0; oc < old->cols; oc++)
+				for (int oc = 0; oc < old->attributes.count; oc++)
 				{
-					if (streq(old->columns[oc], colname))
+					LogicalMessageAttribute *oldAttr = &(old->attributes.array[oc]);
+					if (streq(oldAttr->attname, attr->attname))
 					{
 						/* only works because old->values.count == 1 */
 						LogicalMessageValue *oldValue =
-							&(old->values.array[0].array[v]);
+							&(old->values.array[0].array[oc]);
 
 						if (LogicalMessageValueEq(oldValue, value))
 						{
@@ -2206,16 +2270,25 @@ stream_write_update(FILE *out, LogicalMessageUpdate *update)
 
 				if (!skip)
 				{
-					appendPQExpBuffer(buf, "%s%s = $%d",
-									  first ? "" : ", ",
-									  colname,
-									  ++pos);
-
-					if (!stream_add_value_in_json_array(value, jsArray))
+					if (attr->isgenerated)
 					{
-						/* errors have already been logged */
-						destroyPQExpBuffer(buf);
-						return false;
+						appendPQExpBuffer(buf, "%s%s = DEFAULT",
+										  first ? "" : ", ",
+										  attr->attname);
+					}
+					else
+					{
+						appendPQExpBuffer(buf, "%s%s = $%d",
+										  first ? "" : ", ",
+										  attr->attname,
+										  ++pos);
+
+						if (!stream_add_value_in_json_array(value, jsArray))
+						{
+							/* errors have already been logged */
+							destroyPQExpBuffer(buf);
+							return false;
+						}
 					}
 
 					if (first)
@@ -2235,14 +2308,15 @@ stream_write_update(FILE *out, LogicalMessageUpdate *update)
 			/* now loop over column values for this VALUES row */
 			for (int v = 0; v < values->cols; v++)
 			{
+				LogicalMessageAttribute *attr = &(old->attributes.array[v]);
 				LogicalMessageValue *value = &(values->array[v]);
 
-				if (old->cols <= v)
+				if (old->attributes.count <= v)
 				{
 					log_error("Failed to write UPDATE statement with more "
 							  "VALUES (%d) than COLUMNS (%d)",
 							  values->cols,
-							  old->cols);
+							  old->attributes.count);
 					destroyPQExpBuffer(buf);
 					return false;
 				}
@@ -2255,13 +2329,13 @@ stream_write_update(FILE *out, LogicalMessageUpdate *update)
 					 */
 					appendPQExpBuffer(buf, "%s%s IS NULL",
 									  v > 0 ? " and " : "",
-									  old->columns[v]);
+									  attr->attname);
 				}
 				else
 				{
 					appendPQExpBuffer(buf, "%s%s = $%d",
 									  v > 0 ? " and " : "",
-									  old->columns[v],
+									  attr->attname,
 									  ++pos);
 
 					if (!stream_add_value_in_json_array(value, jsArray))
@@ -2334,13 +2408,14 @@ stream_write_delete(FILE *out, LogicalMessageDelete *delete)
 			for (int v = 0; v < values->cols; v++)
 			{
 				LogicalMessageValue *value = &(values->array[v]);
+				LogicalMessageAttribute *attr = &(old->attributes.array[v]);
 
-				if (old->cols <= v)
+				if (old->attributes.count <= v)
 				{
 					log_error("Failed to write DELETE statement with more "
 							  "VALUES (%d) than COLUMNS (%d)",
 							  values->cols,
-							  old->cols);
+							  old->attributes.count);
 					destroyPQExpBuffer(buf);
 					return false;
 				}
@@ -2353,13 +2428,13 @@ stream_write_delete(FILE *out, LogicalMessageDelete *delete)
 					 */
 					appendPQExpBuffer(buf, "%s%s IS NULL",
 									  v > 0 ? " and " : "",
-									  old->columns[v]);
+									  attr->attname);
 				}
 				else
 				{
 					appendPQExpBuffer(buf, "%s%s = $%d",
 									  v > 0 ? " and " : "",
-									  old->columns[v],
+									  attr->attname,
 									  ++pos);
 
 					if (!stream_add_value_in_json_array(value, jsArray))
@@ -2609,3 +2684,292 @@ LogicalMessageValueEq(LogicalMessageValue *a, LogicalMessageValue *b)
 	/* makes compiler happy */
 	return false;
 }
+
+
+/*
+ * Identifiers such as schema, table, column comes from various
+ * sources(e.g. wal2json, test_decoding and source catalog) and some of them
+ * already escapes identifiers and few don't.
+ * We need to check if the identifier is already quoted or not before
+ * escaping it.
+ * Whatever we are here is not a fool proof escaping mechanism, but a best
+ * effort to make sure that the identifiers are normalized by quoting them
+ * if it is not already quoted.
+ *
+ * Here is an example:
+ * foo -> "foo"
+ * "foo" -> "foo"
+ * foo"bar -> "foo"bar"
+ * "foo -> ""foo"
+ *
+ * The goal of this normalization is to make sure that the identifiers are
+ * comparable in the context of Hash Table.
+ */
+#define NORMALIZED_PG_NAMEDATA_COPY(dst, src) \
+	{ \
+		int len = strlen(src); \
+		if (src[0] == '"' && src[len - 1] == '"') \
+		{ \
+			strlcpy(dst, src, PG_NAMEDATALEN); \
+		} \
+		else \
+		{ \
+			sformat(dst, PG_NAMEDATALEN, "\"%s\"", src); \
+		} \
+	}
+
+/*
+ * lookupGeneratedColumnsForTable lookup the generated columns set for the given
+ * table "nspname.relname".
+ *
+ * Returns a GeneratedColumnSet if the table has generated columns, NULL
+ * otherwise.
+ *
+ * NOTE: There is no error condition, if the cache is NULL, it means that we
+ * don't have any generated columns in the catalog.
+ */
+static GeneratedColumnSet *
+lookupGeneratedColumnsForTable(GeneratedColumnsCache *cache,
+							   const char *nspname,
+							   const char *relname)
+{
+	/*
+	 * NULL cache means that we don't have any generated columns in the
+	 * catalog.
+	 */
+	if (cache == NULL)
+	{
+		return NULL;
+	}
+
+	GeneratedColumnsCache *item = NULL;
+
+	GeneratedColumnsCache_Lookup key = { 0 };
+
+	NORMALIZED_PG_NAMEDATA_COPY(key.nspname, nspname);
+	NORMALIZED_PG_NAMEDATA_COPY(key.relname, relname);
+
+	HASH_FIND(hh, cache, &key, sizeof(GeneratedColumnsCache_Lookup), item);
+
+	if (item == NULL)
+	{
+		return NULL;
+	}
+
+	if (item->columns == NULL)
+	{
+		log_error("BUG: Table \"%s.%s\" is in the cache but columns are NULL",
+				  nspname, relname);
+		return NULL;
+	}
+
+	log_trace("Table \"%s.%s\" has generated columns", nspname, relname);
+
+	return item->columns;
+}
+
+
+/*
+ * isGeneratedColumn checks whether the given "attname" is a generated column.
+ *
+ * Returns true if the column is generated, false otherwise.
+ *
+ * NOTE: There is no error condition, if the columns is NULL, it means that we
+ * don't have any generated columns in the catalog.
+ */
+static bool
+isGeneratedColumn(GeneratedColumnSet *columns, const char *attname)
+{
+	char attnameNormalized[PG_NAMEDATALEN] = { 0 };
+
+	NORMALIZED_PG_NAMEDATA_COPY(attnameNormalized, attname);
+
+	GeneratedColumnSet *generatedColumns = NULL;
+
+	HASH_FIND_STR(columns, attnameNormalized, generatedColumns);
+
+	if (generatedColumns != NULL)
+	{
+		log_trace("Column \"%s\" is generated", attnameNormalized);
+
+		return true;
+	}
+
+	return false;
+}
+
+
+/*
+ * prepareGeneratedColumnsCache_hook is a callback function that populates the
+ * generated columns cache from the catalog.
+ */
+static bool
+prepareGeneratedColumnsCache_hook(void *ctx, SourceTable *table)
+{
+	StreamSpecs *specs = (StreamSpecs *) ctx;
+	StreamContext *privateContext = &(specs->private);
+	DatabaseCatalog *sourceDB = specs->sourceDB;
+
+	if (!catalog_s_table_fetch_attrs(sourceDB, table))
+	{
+		log_error("Failed to fetch attributes for table \"%s\".%s",
+				  table->nspname, table->relname);
+		return false;
+	}
+
+	GeneratedColumnsCache *item = (GeneratedColumnsCache *)
+								  calloc(1, sizeof(GeneratedColumnsCache));
+
+	if (item == NULL)
+	{
+		log_error(ALLOCATION_FAILED_ERROR);
+		return false;
+	}
+
+	for (int i = 0; i < table->attributes.count; i++)
+	{
+		SourceTableAttribute *attr = &(table->attributes.array[i]);
+
+		if (attr->attisgenerated)
+		{
+			/* Add a generated column to the GeneratedColumnSet */
+			GeneratedColumnSet *generatedColumn = (GeneratedColumnSet *)
+												  calloc(1, sizeof(GeneratedColumnSet));
+			if (generatedColumn == NULL)
+			{
+				log_error(ALLOCATION_FAILED_ERROR);
+				return false;
+			}
+
+			NORMALIZED_PG_NAMEDATA_COPY(generatedColumn->attname, attr->attname);
+			HASH_ADD_STR(item->columns, attname, generatedColumn);
+		}
+	}
+
+
+	NORMALIZED_PG_NAMEDATA_COPY(item->nspname, table->nspname);
+	NORMALIZED_PG_NAMEDATA_COPY(item->relname, table->relname);
+
+	/*
+	 * Prepare keylen as per https://troydhanson.github.io/uthash/userguide.html#_compound_keys
+	 */
+	unsigned keylen = offsetof(GeneratedColumnsCache, relname) + /* offset of last key field */
+					  sizeof(item->relname) -    /* size of last key field */
+					  offsetof(GeneratedColumnsCache, nspname); /* offset of first key field */
+
+	/* Add the table to the GeneratedColumnsCache. */
+	HASH_ADD(hh,
+			 privateContext->generatedColumnsCache,
+			 nspname,
+			 keylen,
+			 item);
+
+	return true;
+}
+
+
+/*
+ * prepareGeneratedColumnsCache fills-in the cache with the tables having
+ * generated columns.
+ */
+static bool
+prepareGeneratedColumnsCache(StreamSpecs *specs)
+{
+	/*
+	 * TODO: GeneratedColumn must be retrieved from the target catalog
+	 * because the schema of the target can be different from the source.
+	 */
+	if (!catalog_iter_s_table_generated_columns(specs->sourceDB,
+												specs,
+												&prepareGeneratedColumnsCache_hook))
+	{
+		log_error("Failed to prepare a generated column cache for our catalog,"
+				  "see above for details");
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * markGeneratedColumnsFromTransaction marks the generated columns in the
+ * transaction.
+ */
+static bool
+markGeneratedColumnsFromTransaction(GeneratedColumnsCache *cache,
+									LogicalTransaction *txn)
+{
+	LogicalTransactionStatement *stmt = txn->first;
+
+	for (; stmt != NULL; stmt = stmt->next)
+	{
+		if (!markGeneratedColumnsFromStatement(cache, stmt))
+		{
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
+/*
+ * markGeneratedColumnsFromStatement marks the generated columns in the
+ * given statement after looking up the cache.
+ */
+static bool
+markGeneratedColumnsFromStatement(GeneratedColumnsCache *cache,
+								  LogicalTransactionStatement *stmt)
+{
+	LogicalMessageTupleArray *columns = NULL;
+	const char *nspname = NULL;
+	const char *relname = NULL;
+
+	if (stmt->action == STREAM_ACTION_INSERT)
+	{
+		columns = &(stmt->stmt.insert.new);
+		nspname = stmt->stmt.insert.table.nspname;
+		relname = stmt->stmt.insert.table.relname;
+	}
+	else if (stmt->action == STREAM_ACTION_UPDATE)
+	{
+		columns = &(stmt->stmt.update.new);
+		nspname = stmt->stmt.update.table.nspname;
+		relname = stmt->stmt.update.table.relname;
+	}
+	else
+	{
+		/*
+		 * Only INSERT and UPDATE statements can update the table
+		 * generated columns.
+		 */
+		return true;
+	}
+
+	GeneratedColumnSet *generatedColumns = lookupGeneratedColumnsForTable(cache, nspname,
+																		  relname);
+
+	if (generatedColumns == NULL)
+	{
+		/* no generated columns in this table */
+		return true;
+	}
+
+	for (int i = 0; i < columns->count; i++)
+	{
+		LogicalMessageTuple *tuple = &(columns->array[i]);
+
+		for (int c = 0; c < tuple->attributes.count; c++)
+		{
+			LogicalMessageAttribute *attr = &(tuple->attributes.array[c]);
+
+			if (isGeneratedColumn(generatedColumns, attr->attname))
+			{
+				attr->isgenerated = true;
+			}
+		}
+	}
+
+	return true;
+}
