@@ -12,11 +12,24 @@
 #include <sys/ipc.h>
 #include <sys/sem.h>
 
+/*
+ * pgcopydb uses semaphores as a locking mechanism protecting a critical
+ * section, where a single worker/process is expected at any time.
+ *
+ * Some parts of the code might use the semaphore in a re-entrant way, for
+ * instance SQLite iterator query where the caller hook function then runs a
+ * query at each step of the iteration.
+ */
 typedef struct Semaphore
 {
 	int semId;
 	int initValue;
 	pid_t owner;
+
+	bool reentrant;
+	int depth;
+
+	bool debug;
 } Semaphore;
 
 
