@@ -120,7 +120,7 @@ find_pg_commands(PostgresPaths *pgPaths)
 
 
 /*
- * set_postgres_commands sets the rest of the Postgres commands that pgcyopdb
+ * set_postgres_commands sets the rest of the Postgres commands that pgcopydb
  * needs from knowing the pgPaths->psql absolute location already.
  */
 void
@@ -129,6 +129,7 @@ set_postgres_commands(PostgresPaths *pgPaths)
 	path_in_same_directory(pgPaths->psql, "pg_dump", pgPaths->pg_dump);
 	path_in_same_directory(pgPaths->psql, "pg_dumpall", pgPaths->pg_dumpall);
 	path_in_same_directory(pgPaths->psql, "pg_restore", pgPaths->pg_restore);
+	path_in_same_directory(pgPaths->psql, "vacuumdb", pgPaths->vacuumdb);
 }
 
 
@@ -357,13 +358,13 @@ typedef struct DumpExtensionNamespaceContext
 
 
 /*
- * Call pg_dump and get the given section of the dump into the target file.
+ * Call pg_dump and get the given pre-data and post-data sections of
+ * the dump into the target file.
  */
 bool
 pg_dump_db(PostgresPaths *pgPaths,
 		   ConnStrings *connStrings,
 		   const char *snapshot,
-		   const char *section,
 		   SourceFilters *filters,
 		   DatabaseCatalog *filtersDB,
 		   const char *filename)
@@ -401,8 +402,14 @@ pg_dump_db(PostgresPaths *pgPaths,
 		args[argsIndex++] = (char *) snapshot;
 	}
 
-	args[argsIndex++] = "--section";
-	args[argsIndex++] = (char *) section;
+	/*
+	 * To address a migration issue (https://github.com/dimitri/pgcopydb/issues/760),
+	 * we need to dump the pre-data and post-data sections together in a single file.
+	 * Using --schema-only instead is not an option because it does not include
+	 * the necessary large object metadata.
+	 */
+	args[argsIndex++] = "--section=pre-data";
+	args[argsIndex++] = "--section=post-data";
 
 	/* apply [include-only-schema] filtering */
 	for (int i = 0; i < filters->includeOnlySchemaList.count; i++)
@@ -428,7 +435,7 @@ pg_dump_db(PostgresPaths *pgPaths,
 		char *nspname = filters->excludeSchemaList.array[i].nspname;
 
 		/* check that we still have room for --exclude-schema args */
-		if (PG_CMD_MAX_ARG < (argsIndex + 2))
+		if (PG_CMD_MAX_ARG < (argsIndex + 6))
 		{
 			log_error("Failed to call pg_dump, too many exclude-schema entries: "
 					  "argsIndex %d > %d",
@@ -488,6 +495,86 @@ pg_dump_db(PostgresPaths *pgPaths,
 }
 
 
+/*
+ * Call `vacuumdb --analyze-only --jobs ${table-jobs}`
+ */
+bool
+pg_vacuumdb_analyze_only(PostgresPaths *pgPaths, ConnStrings *connStrings, int jobs)
+{
+	char *args[16];
+	int argsIndex = 0;
+
+	char command[BUFSIZE] = { 0 };
+
+	char *PGPASSWORD = NULL;
+	bool pgpassword_found_in_env = env_exists("PGPASSWORD");
+
+	if (!env_exists("PGCONNECT_TIMEOUT"))
+	{
+		setenv("PGCONNECT_TIMEOUT", POSTGRES_CONNECT_TIMEOUT, 1);
+	}
+
+	/* override PGPASSWORD environment variable if the pguri contains one */
+	if (connStrings->safeSourcePGURI.password != NULL)
+	{
+		if (pgpassword_found_in_env && !get_env_dup("PGPASSWORD", &PGPASSWORD))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+		setenv("PGPASSWORD", connStrings->safeSourcePGURI.password, 1);
+	}
+
+	args[argsIndex++] = (char *) pgPaths->vacuumdb;
+	args[argsIndex++] = "--analyze-only";
+	args[argsIndex++] = "--jobs";
+	args[argsIndex++] = intToString(jobs).strValue;
+	args[argsIndex++] = "--dbname";
+	args[argsIndex++] = (char *) connStrings->safeSourcePGURI.pguri;
+
+	args[argsIndex] = NULL;
+
+	/*
+	 * We do not want to call setsid() when running vacuumdb.
+	 */
+	Program program = { 0 };
+
+	(void) initialize_program(&program, args, false);
+	program.processBuffer = &processBufferCallback;
+
+	/* log the exact command line we're using */
+	int commandSize = snprintf_program_command_line(&program, command, BUFSIZE);
+
+	if (commandSize >= BUFSIZE)
+	{
+		/* we only display the first BUFSIZE bytes of the real command */
+		log_info("%s...", command);
+	}
+	else
+	{
+		log_info("%s", command);
+	}
+
+	(void) execute_subprogram(&program);
+
+	/* make sure to reset the environment PGPASSWORD if we edited it */
+	if (pgpassword_found_in_env &&
+		connStrings->safeSourcePGURI.password != NULL)
+	{
+		setenv("PGPASSWORD", PGPASSWORD, 1);
+	}
+
+	if (program.returnCode != 0)
+	{
+		log_error("Failed to run vacuumdb: exit code %d", program.returnCode);
+
+		return false;
+	}
+
+	return true;
+}
+
+
 /*
  * Call pg_dump and get the given section of the dump into the target file.
  */
@@ -816,6 +903,16 @@ pg_restore_db(PostgresPaths *pgPaths,
 	args[argsIndex++] = "--dbname";
 	args[argsIndex++] = (char *) connStrings->safeTargetPGURI.pguri;
 
+	const char *sectionOption = postgresRestoreSectionToString(options.section);
+	if (sectionOption == NULL)
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	args[argsIndex++] = "--section";
+	args[argsIndex++] = (char *) sectionOption;
+
 	if (options.jobs == 1)
 	{
 		args[argsIndex++] = "--single-transaction";
@@ -932,14 +1029,13 @@ pg_restore_db(PostgresPaths *pgPaths,
 
 
 /*
- * pg_restore_list runs the command pg_restore -f- -l on the given custom
- * format dump file and returns an array of pg_dump archive objects.
+ * pg_restore_list runs the command pg_restore -f listFilename --list on the
+ * given custom format dump file.
  */
 bool
 pg_restore_list(PostgresPaths *pgPaths,
 				const char *restoreFilename,
-				const char *listFilename,
-				ArchiveContentArray *archive)
+				const char *listFilename)
 {
 	char *args[PG_CMD_MAX_ARG];
 	int argsIndex = 0;
@@ -977,7 +1073,207 @@ pg_restore_list(PostgresPaths *pgPaths,
 		return false;
 	}
 
-	if (!parse_archive_list(listFilename, archive))
+	return true;
+}
+
+
+/*
+ * archive_iter_toc reads a pg_dump archive TOC file as obtained with the
+ * pg_restore --list command, and iterates through the archive TOC entries.
+ */
+bool
+archive_iter_toc(const char *filename, void *context, ArchiveTOCFun *callback)
+{
+	ArchiveTOCIterator *iter =
+		(ArchiveTOCIterator *) calloc(1, sizeof(ArchiveTOCIterator));
+
+	if (iter == NULL)
+	{
+		log_error(ALLOCATION_FAILED_ERROR);
+		return false;
+	}
+
+	iter->filename = filename;
+
+	if (!archive_iter_toc_init(iter))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	for (;;)
+	{
+		if (!archive_iter_toc_next(iter))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+
+		ArchiveContentItem *item = iter->item;
+
+		if (item == NULL)
+		{
+			if (!archive_iter_toc_finish(iter))
+			{
+				/* errors have already been logged */
+				return false;
+			}
+
+			break;
+		}
+
+		if (!(*callback)(context, item))
+		{
+			log_error("Failed to iterate over lines of file \"%s\", "
+					  "see above for details",
+					  iter->filename);
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
+/*
+ * archive_iter_toc_init initializes an Iterator over a Postgres archive TOC
+ * file.
+ */
+bool
+archive_iter_toc_init(ArchiveTOCIterator *iter)
+{
+	iter->fileIterator =
+		(FileLinesIterator *) calloc(1, sizeof(FileLinesIterator));
+
+	if (iter->fileIterator == NULL)
+	{
+		log_error(ALLOCATION_FAILED_ERROR);
+		return false;
+	}
+
+	iter->fileIterator->filename = iter->filename;
+
+	/*
+	 * An archive TOC entry has a fixed format that looks like the following:
+	 *
+	 * ahprintf(AH, "%d; %u %u %s %s %s %s\n", te->dumpId,
+	 *          te->catalogId.tableoid, te->catalogId.oid, te->desc,
+	 *          sanitized_schema, sanitized_name, sanitized_owner);
+	 *
+	 * The %s parts are PG_NAMEDATALEN. The overall line fits into BUFSIZE.
+	 *
+	 * Numbers require up to 10 chars (digits) to be printed, and
+	 * PG_NAMEDATALEN is 64 characters, but we need to double that for quoting
+	 * rules. Add in the semi-colon. And the desc is less than 32 characters,
+	 * the longest known being "PUBLICATION TABLES IN SCHEMA" at 30. Let's make
+	 * it 64 for headroom:
+	 *
+	 * 3 * (10 + 1) + 1 + 3 * (2 * 64) + 64 = 482 chars
+	 *
+	 * BUFSIZE is 1024 bytes.
+	 */
+	iter->fileIterator->bufsize = BUFSIZE;
+
+	if (!file_iter_lines_init(iter->fileIterator))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	iter->item = (ArchiveContentItem *) calloc(1, sizeof(ArchiveContentItem));
+
+	if (iter->item == NULL)
+	{
+		log_error(ALLOCATION_FAILED_ERROR);
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * archive_iter_toc_next fetches the next archive TOC item.
+ */
+bool
+archive_iter_toc_next(ArchiveTOCIterator *iter)
+{
+	/* read next line using the file iterator low-level API */
+	bool skippingEmptyLinesAndComments = true;
+
+	while (skippingEmptyLinesAndComments)
+	{
+		if (!file_iter_lines_next(iter->fileIterator))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+
+		if (iter->fileIterator->line == NULL)
+		{
+			iter->item = NULL;
+			return true;
+		}
+
+		/* skip empty lines and lines that start with a semi-colon (comment) */
+		char *line = iter->fileIterator->line;
+
+		skippingEmptyLinesAndComments = (*line == '\0' || *line == ';');
+	}
+
+	/* remove end-of-line \n character in our line buffer */
+	size_t len = strlen(iter->fileIterator->line);
+
+	if (iter->fileIterator->line[len - 1] == '\n')
+	{
+		iter->fileIterator->line[len - 1] = '\0';
+	}
+	else
+	{
+		/* if that's not the last line, then we got a partial read */
+		if (feof(iter->fileIterator->stream) != 0)
+		{
+			log_error("Failed to parse archive TOC file \"%s\": partial read",
+					  iter->filename);
+			return false;
+		}
+	}
+
+	/* reset our memory area before re-use */
+	ArchiveContentItem empty = { 0 };
+	*(iter->item) = empty;
+
+	/* now parse the line */
+	if (!parse_archive_list_entry(iter->item, iter->fileIterator->line))
+	{
+		/* errors have already been logged  */
+		return false;
+	}
+
+	log_trace("archive_iter_toc_next: %u; %u %u %s %s",
+			  iter->item->dumpId,
+			  iter->item->catalogOid,
+			  iter->item->objectOid,
+			  iter->item->description,
+			  iter->item->restoreListName);
+
+	if (iter->item->desc == ARCHIVE_TAG_UNKNOWN ||
+		IS_EMPTY_STRING_BUFFER(iter->item->description))
+	{
+		log_warn("Failed to parse desc \"%s\"", iter->fileIterator->line);
+	}
+
+	return true;
+}
+
+
+/*
+ * archive_iter_toc_finish closes the iterator.
+ */
+bool
+archive_iter_toc_finish(ArchiveTOCIterator *iter)
+{
+	if (!file_iter_lines_finish(iter->fileIterator))
 	{
 		/* errors have already been logged */
 		return false;
@@ -1044,6 +1340,13 @@ struct ArchiveItemDescMapping pgRestoreDescriptionArray[] = {
 	INSERT_MAPPING(ARCHIVE_TAG_INDEX, "INDEX"),
 	INSERT_MAPPING(ARCHIVE_TAG_LANGUAGE, "LANGUAGE"),
 	INSERT_MAPPING(ARCHIVE_TAG_LARGE_OBJECT, "LARGE OBJECT"),
+
+	/*
+	 * MATERIALIZED VIEW DATA should come before MATERIALIZED VIEW, otherwise
+	 * the strncmp will match the first part of the string and misidentify the
+	 * MATERIALIZED VIEW DATA as MATERIALIZED VIEW.
+	 */
+	INSERT_MAPPING(ARCHIVE_TAG_REFRESH_MATERIALIZED_VIEW, "MATERIALIZED VIEW DATA"),
 	INSERT_MAPPING(ARCHIVE_TAG_MATERIALIZED_VIEW, "MATERIALIZED VIEW"),
 	INSERT_MAPPING(ARCHIVE_TAG_OPERATOR_CLASS, "OPERATOR CLASS"),
 	INSERT_MAPPING(ARCHIVE_TAG_OPERATOR_FAMILY, "OPERATOR FAMILY"),
@@ -1055,7 +1358,6 @@ struct ArchiveItemDescMapping pgRestoreDescriptionArray[] = {
 				   "PUBLICATION TABLES IN SCHEMA"),
 	INSERT_MAPPING(ARCHIVE_TAG_PUBLICATION_TABLE, "PUBLICATION TABLE"),
 	INSERT_MAPPING(ARCHIVE_TAG_PUBLICATION, "PUBLICATION"),
-	INSERT_MAPPING(ARCHIVE_TAG_REFRESH_MATERIALIZED_VIEW, "REFRESH MATERIALIZED VIEW"),
 	INSERT_MAPPING(ARCHIVE_TAG_ROW_SECURITY, "ROW SECURITY"),
 	INSERT_MAPPING(ARCHIVE_TAG_RULE, "RULE"),
 	INSERT_MAPPING(ARCHIVE_TAG_SCHEMA, "SCHEMA"),
@@ -1374,15 +1676,9 @@ tokenize_archive_list_entry(ArchiveToken *token)
 		char *ptr = line;
 
 		/* advance ptr as long as *ptr is a digit */
-		for (; ptr != NULL && isdigit(*ptr); ptr++)
+		for (; isdigit(*ptr); ptr++)
 		{ }
 
-		if (ptr == NULL)
-		{
-			log_error("Failed to tokenize Archive Item line: %s", line);
-			return false;
-		}
-
 		int len = ptr - line + 1;
 		size_t size = len + 1;
 		char *buf = (char *) calloc(size, sizeof(char));
