@@ -336,10 +336,21 @@ typedef struct CopyArgs
 	char *logCommand;
 	bool truncate;
 	bool freeze;
-	uint64_t bytesTransmitted;
+	bool useCopyBinary;
 } CopyArgs;
 
-bool pg_copy(PGSQL *src, PGSQL *dst, CopyArgs *args);
+
+typedef struct CopyStats
+{
+	uint64_t startTime;
+	uint64_t bytesTransmitted;
+} CopyStats;
+
+typedef bool (CopyStatsCallback)(void *context, CopyStats *stats);
+
+bool pg_copy(PGSQL *src, PGSQL *dst,
+			 CopyArgs *args, CopyStats *stats,
+			 void *context, CopyStatsCallback *callback);
 
 bool pg_copy_from_stdin(PGSQL *pgsql, const char *qname);
 bool pg_copy_row_from_stdin(PGSQL *pgsql, char *fmt, ...);
@@ -367,10 +378,10 @@ bool pg_copy_large_object(PGSQL *src,
 #define PG_LSN_MAXLENGTH 18
 
 /*
- * TimeLineHistoryEntry is taken from Postgres definitions and adapted to
+ * TimelineHistoryEntry is taken from Postgres definitions and adapted to
  * client-size code where we don't have all the necessary infrastruture. In
  * particular we don't define a XLogRecPtr data type nor do we define a
- * TimeLineID data type.
+ * TimelineID data type.
  *
  * Zero is used indicate an invalid pointer. Bootstrap skips the first possible
  * WAL segment, initializing the first WAL page at WAL segment size, so no XLOG
@@ -379,26 +390,14 @@ bool pg_copy_large_object(PGSQL *src,
 #define InvalidXLogRecPtr 0
 #define XLogRecPtrIsInvalid(r) ((r) == InvalidXLogRecPtr)
 
-#define PGCOPYDB_MAX_TIMELINES 1024
-#define PGCOPYDB_MAX_TIMELINE_CONTENT (1024 * 1024)
-
-typedef struct TimeLineHistoryEntry
+typedef struct TimelineHistoryEntry
 {
 	uint32_t tli;
 	uint64_t begin;         /* inclusive */
 	uint64_t end;           /* exclusive, InvalidXLogRecPtr means infinity */
-} TimeLineHistoryEntry;
+} TimelineHistoryEntry;
 
 
-typedef struct TimeLineHistory
-{
-	int count;
-	TimeLineHistoryEntry history[PGCOPYDB_MAX_TIMELINES];
-
-	char filename[MAXPGPATH];
-	char content[PGCOPYDB_MAX_TIMELINE_CONTENT];
-} TimeLineHistory;
-
 /*
  * The IdentifySystem contains information that is parsed from the
  * IDENTIFY_SYSTEM replication command, and then the TIMELINE_HISTORY result.
@@ -409,12 +408,10 @@ typedef struct IdentifySystem
 	uint32_t timeline;
 	char xlogpos[PG_LSN_MAXLENGTH];
 	char dbname[NAMEDATALEN];
-	TimeLineHistory timelines;
+	TimelineHistoryEntry currentTimeline;
+	char timelineHistoryFilename[MAXPGPATH];
 } IdentifySystem;
 
-bool pgsql_identify_system(PGSQL *pgsql, IdentifySystem *system);
-bool parseTimeLineHistory(const char *filename, const char *content,
-						  IdentifySystem *system);
 
 /*
  * Logical Decoding support.
@@ -470,6 +467,7 @@ typedef struct LogicalStreamClient
 {
 	PGSQL pgsql;
 	IdentifySystem system;
+	char cdcPathDir[MAXPGPATH];
 
 	char slotName[NAMEDATALEN];
 
@@ -532,6 +530,8 @@ bool pgsql_stream_logical(LogicalStreamClient *client,
 
 bool RetrieveWalSegSize(LogicalStreamClient *client);
 
+bool pgsql_get_block_size(PGSQL *pgsql, int *blockSize);
+
 bool pgsql_replication_origin_oid(PGSQL *pgsql, char *nodeName, uint32_t *oid);
 bool pgsql_replication_origin_create(PGSQL *pgsql, char *nodeName);
 bool pgsql_replication_origin_drop(PGSQL *pgsql, char *nodeName);
@@ -566,8 +566,8 @@ bool pgsql_configuration_exists(PGSQL *pgsql, const char *setconfig, bool *exist
 
 bool pgsql_table_exists(PGSQL *pgsql,
 						uint32_t oid,
-						const char *relname,
 						const char *nspname,
+						const char *relname,
 						bool *exists);
 
 bool pgsql_current_wal_flush_lsn(PGSQL *pgsql, uint64_t *lsn);
