@@ -128,7 +128,7 @@ read_pidfile(const char *pidfile, pid_t *pid)
 
 	*pid = pidnum;
 
-	if (pid <= 0)
+	if (*pid <= 0)
 	{
 		log_debug("Read negative pid %d in file \"%s\", removing it",
 				  *pid, pidfile);
