@@ -36,6 +36,15 @@ static bool copydb_table_array_as_json(DatabaseCatalog *sourceDB,
 									   JSON_Object *jsobj,
 									   const char *key);
 
+static bool copydb_table_in_progress_as_json(DatabaseCatalog *sourceDB,
+											 SourceTableArray *tableInProgress,
+											 JSON_Object *jsobj,
+											 const char *key);
+
+static bool copydb_index_in_progress_as_json(SourceIndexArray *indexInProgress,
+											 JSON_Object *jsobj,
+											 const char *key);
+
 static bool copydb_index_array_as_json(DatabaseCatalog *sourceDB,
 									   JSON_Object *jsobj,
 									   const char *key);
@@ -119,7 +128,6 @@ copydb_prepare_schema_json_file(CopyDataSpec *copySpecs)
 	}
 
 	json_free_serialized_string(serialized_string);
-	json_value_free(js);
 
 	return true;
 }
@@ -243,6 +251,43 @@ copydb_table_array_as_json(DatabaseCatalog *sourceDB,
 }
 
 
+/*
+ * copydb_table_in_progress_as_json prepares the given SourceTableArray as a
+ * JSON array of objects within the given JSON_Object.
+ */
+static bool
+copydb_table_in_progress_as_json(DatabaseCatalog *sourceDB,
+								 SourceTableArray *tableInProgress,
+								 JSON_Object *jsobj,
+								 const char *key)
+{
+	JSON_Value *jsTables = json_value_init_array();
+	JSON_Array *jsTableArray = json_value_get_array(jsTables);
+
+	TableContext context = {
+		.sourceDB = sourceDB,
+		.jsTableArray = jsTableArray
+	};
+
+	for (int i = 0; i < tableInProgress->count; i++)
+	{
+		SourceTable *table = &(tableInProgress->array[i]);
+
+		if (!copydb_table_array_as_json_hook(&context, table))
+		{
+			log_error("Failed to populate the JSON array of tables in progress, "
+					  "see above for details");
+			return false;
+		}
+	}
+
+	/* attach the JSON array to the main JSON object under the provided key */
+	json_object_set_value(jsobj, key, jsTables);
+
+	return true;
+}
+
+
 /*
  * copydb_table_array_as_json_hook is an iterator callback function.
  */
@@ -401,6 +446,37 @@ copydb_index_array_as_json(DatabaseCatalog *sourceDB,
 }
 
 
+/*
+ * copydb_index_in_progress_as_json prepares the given SourceIndexArray as a
+ * JSON array of objects within the given JSON_Object.
+ */
+static bool
+copydb_index_in_progress_as_json(SourceIndexArray *indexInProgress,
+								 JSON_Object *jsobj,
+								 const char *key)
+{
+	JSON_Value *jsIndexes = json_value_init_array();
+	JSON_Array *jsIndexArray = json_value_get_array(jsIndexes);
+
+	for (int i = 0; i < indexInProgress->count; i++)
+	{
+		SourceIndex *index = &(indexInProgress->array[i]);
+
+		if (!copydb_index_array_as_json_hook(jsIndexArray, index))
+		{
+			log_error("Failed to populate the JSON array of indexs in progress, "
+					  "see above for details");
+			return false;
+		}
+	}
+
+	/* attach the JSON array to the main JSON object under the provided key */
+	json_object_set_value(jsobj, key, jsIndexes);
+
+	return true;
+}
+
+
 /*
  * copydb_index_array_as_json_hook is an iterator callback function.
  */
@@ -569,10 +645,12 @@ copydb_update_progress(CopyDataSpec *copySpecs, CopyProgress *progress)
 	/* count table in progress, table done */
 	progress->tableDoneCount = done.table;
 	progress->tableInProgress.count = 0;
+	progress->tableInProgress.capacity = ARRAY_CAPACITY_INCREMENT;
 
 	/* we can't have more table in progress than tableJobs */
 	progress->tableInProgress.array =
-		(SourceTable *) calloc(copySpecs->tableJobs, sizeof(SourceTable));
+		(SourceTable *) calloc(progress->tableInProgress.capacity,
+							   sizeof(SourceTable));
 
 	if (progress->tableInProgress.array == NULL)
 	{
@@ -581,8 +659,9 @@ copydb_update_progress(CopyDataSpec *copySpecs, CopyProgress *progress)
 	}
 
 	progress->tableSummaryArray.count = 0;
+	progress->tableSummaryArray.capacity = ARRAY_CAPACITY_INCREMENT;
 	progress->tableSummaryArray.array =
-		(CopyTableSummary *) calloc(copySpecs->tableJobs,
+		(CopyTableSummary *) calloc(progress->tableSummaryArray.capacity,
 									sizeof(CopyTableSummary));
 
 	if (progress->tableSummaryArray.array == NULL)
@@ -607,10 +686,12 @@ copydb_update_progress(CopyDataSpec *copySpecs, CopyProgress *progress)
 	/* count index in progress, index done */
 	progress->indexDoneCount = done.index;
 	progress->indexInProgress.count = 0;
+	progress->indexInProgress.capacity = ARRAY_CAPACITY_INCREMENT;
 
 	/* we can't have more index in progress than indexJobs */
 	progress->indexInProgress.array =
-		(SourceIndex *) calloc(copySpecs->indexJobs, sizeof(SourceIndex));
+		(SourceIndex *) calloc(progress->indexInProgress.capacity,
+							   sizeof(SourceIndex));
 
 	if (progress->indexInProgress.array == NULL)
 	{
@@ -619,8 +700,9 @@ copydb_update_progress(CopyDataSpec *copySpecs, CopyProgress *progress)
 	}
 
 	progress->indexSummaryArray.count = 0;
+	progress->indexSummaryArray.capacity = 0;
 	progress->indexSummaryArray.array =
-		(CopyIndexSummary *) calloc(copySpecs->indexJobs,
+		(CopyIndexSummary *) calloc(progress->indexSummaryArray.capacity,
 									sizeof(CopyIndexSummary));
 
 	if (progress->indexSummaryArray.array == NULL)
@@ -676,6 +758,34 @@ copydb_update_progress_table_hook(void *ctx, SourceTable *table)
 	/*
 	 * Copy the SourceTable struct in-place to the tableInProgress array.
 	 */
+	if (tableInProgress->count == tableInProgress->capacity)
+	{
+		tableInProgress->capacity += ARRAY_CAPACITY_INCREMENT;
+
+		tableInProgress->array =
+			(SourceTable *) realloc(tableInProgress->array,
+									tableInProgress->capacity *
+									sizeof(SourceTable));
+
+		if (tableInProgress->array == NULL)
+		{
+			log_fatal(ALLOCATION_FAILED_ERROR);
+			return false;
+		}
+
+		summaryArray->capacity += ARRAY_CAPACITY_INCREMENT;
+		summaryArray->array =
+			(CopyTableSummary *) realloc(summaryArray->array,
+										 summaryArray->capacity *
+										 sizeof(CopyTableSummary));
+
+		if (summaryArray->array == NULL)
+		{
+			log_fatal(ALLOCATION_FAILED_ERROR);
+			return false;
+		}
+	}
+
 	tableInProgress->array[progress->tableInProgress.count++] = *table;
 	summaryArray->array[progress->tableSummaryArray.count++] = tableSpecs.summary;
 
@@ -737,6 +847,34 @@ copydb_update_progress_index_hook(void *ctx, SourceIndex *index)
 		 * Copy the SourceIndex struct in-place to the indexInProgress
 		 * array.
 		 */
+		if (indexInProgress->count == indexInProgress->capacity)
+		{
+			indexInProgress->capacity += ARRAY_CAPACITY_INCREMENT;
+
+			indexInProgress->array =
+				(SourceIndex *) realloc(indexInProgress->array,
+										indexInProgress->capacity *
+										sizeof(SourceIndex));
+
+			if (indexInProgress->array == NULL)
+			{
+				log_fatal(ALLOCATION_FAILED_ERROR);
+				return false;
+			}
+
+			summaryArrayIdx->capacity += ARRAY_CAPACITY_INCREMENT;
+			summaryArrayIdx->array =
+				(CopyIndexSummary *) realloc(summaryArrayIdx->array,
+											 summaryArrayIdx->capacity *
+											 sizeof(CopyIndexSummary));
+
+			if (summaryArrayIdx->array == NULL)
+			{
+				log_fatal(ALLOCATION_FAILED_ERROR);
+				return false;
+			}
+		}
+
 		indexInProgress->array[progress->indexInProgress.count++] =
 			*index;
 
@@ -761,9 +899,6 @@ copydb_progress_as_json(CopyDataSpec *copySpecs,
 
 	JSON_Object *jsobj = json_value_get_object(js);
 
-	json_object_set_number(jsobj, "table-jobs", copySpecs->tableJobs);
-	json_object_set_number(jsobj, "index-jobs", copySpecs->indexJobs);
-
 	/* table counts */
 	JSON_Value *jsTable = json_value_init_object();
 	JSON_Object *jsTableObj = json_value_get_object(jsTable);
@@ -774,7 +909,10 @@ copydb_progress_as_json(CopyDataSpec *copySpecs,
 	/* in-progress */
 	SourceTableArray *tableArray = &(progress->tableInProgress);
 
-	if (!copydb_table_array_as_json(sourceDB, jsTableObj, "in-progress"))
+	if (!copydb_table_in_progress_as_json(sourceDB,
+										  tableArray,
+										  jsTableObj,
+										  "in-progress"))
 	{
 		/* errors have already been logged */
 		return false;
@@ -810,7 +948,7 @@ copydb_progress_as_json(CopyDataSpec *copySpecs,
 	/* in-progress */
 	SourceIndexArray *indexArray = &(progress->indexInProgress);
 
-	if (!copydb_index_array_as_json(sourceDB, jsIndexObj, "in-progress"))
+	if (!copydb_index_in_progress_as_json(indexArray, jsIndexObj, "in-progress"))
 	{
 		/* errors have already been logged */
 		return false;
