@@ -111,7 +111,7 @@ typedef struct SourceCollation
 typedef struct SourceTableParts
 {
 	int partNumber;
-	int partCount;              /* zero when table is not partitionned */
+	int partCount;              /* zero when table is not partitioned */
 
 	int64_t min;                /* WHERE partKey >= min */
 	int64_t max;                /*   AND partKey  < max */
@@ -157,6 +157,9 @@ typedef struct SourceTable
 	char amname[PG_NAMEDATALEN];
 	char restoreListName[RESTORE_LIST_NAMEDATALEN];
 
+	/* 'r' for ordinary table, 'm' for matview */
+	char relkind;
+
 	int64_t relpages;
 	int64_t reltuples;
 	int64_t bytes;
@@ -175,6 +178,9 @@ typedef struct SourceTable
 	char *attrList;             /* malloc'ed area */
 	SourceTableAttributeArray attributes;
 
+	int64_t bytesEstimate;
+	char bytesEstimatePretty[PG_NAMEDATALEN];
+
 	uint64_t indexCount;
 	uint64_t constraintCount;
 
@@ -333,6 +339,7 @@ typedef struct CatalogSetup
 	char *target_pguri;         /* malloc'ed area */
 	char snapshot[BUFSIZE];
 	uint64_t splitTablesLargerThanBytes;
+	int splitMaxParts;
 	char *filters;              /* malloc'ed area */
 	char plugin[BUFSIZE];
 	char slotName[BUFSIZE];
@@ -412,20 +419,19 @@ bool schema_list_ext_versions(PGSQL *pgsql, ExtensionsVersionsArray *array);
 bool schema_list_collations(PGSQL *pgsql, DatabaseCatalog *catalog);
 
 bool schema_prepare_pgcopydb_table_size(PGSQL *pgsql,
-										SourceFilters *filters, DatabaseCatalog *catalog);
+										SourceFilters *filters,
+										DatabaseCatalog *catalog);
 
 bool schema_list_ordinary_tables(PGSQL *pgsql,
 								 SourceFilters *filters,
+								 bool estimateTableSizes,
 								 DatabaseCatalog *catalog);
 
-bool schema_list_ordinary_tables_without_pk(PGSQL *pgsql,
-											SourceFilters *filters,
-											DatabaseCatalog *catalog);
-
 bool schema_list_partitions(PGSQL *pgsql,
 							DatabaseCatalog *catalog,
 							SourceTable *table,
-							uint64_t partSize);
+							uint64_t partSize,
+							int splitMaxParts);
 
 bool schema_list_sequences(PGSQL *pgsql,
 						   SourceFilters *filters,
