@@ -31,6 +31,7 @@ copydb_copy_snapshot(CopyDataSpec *specs, TransactionSnapshot *snapshot)
 	snapshot->exportedCreateSlotSnapshot = source->exportedCreateSlotSnapshot;
 	snapshot->pguri = strdup(source->pguri);
 	strlcpy(snapshot->snapshot, source->snapshot, sizeof(snapshot->snapshot));
+	snapshot->isReadOnly = source->isReadOnly;
 
 	return true;
 }
@@ -57,12 +58,6 @@ copydb_export_snapshot(TransactionSnapshot *snapshot)
 		return false;
 	}
 
-	if (!pgsql_begin(pgsql))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
 	/*
 	 * As Postgres docs for SET TRANSACTION SNAPSHOT say:
 	 *
@@ -75,11 +70,23 @@ copydb_export_snapshot(TransactionSnapshot *snapshot)
 	 * database to then implement the filtering as JOINs with the Postgres
 	 * catalogs. And even TEMP tables need read-write transaction.
 	 */
-	IsolationLevel level = ISOLATION_SERIALIZABLE;
-	bool readOnly = false;
+	IsolationLevel level = ISOLATION_REPEATABLE_READ;
 	bool deferrable = true;
 
-	if (!pgsql_set_transaction(pgsql, level, readOnly, deferrable))
+	if (!pgsql_is_in_recovery(pgsql, &(snapshot->isReadOnly)))
+	{
+		/* errors have already been logged */
+		(void) pgsql_finish(pgsql);
+		return false;
+	}
+
+	if (!pgsql_begin(pgsql))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	if (!pgsql_set_transaction(pgsql, level, snapshot->isReadOnly, deferrable))
 	{
 		/* errors have already been logged */
 		(void) pgsql_finish(pgsql);
@@ -160,10 +167,9 @@ copydb_set_snapshot(CopyDataSpec *copySpecs)
 		 * catalogs. And even TEMP tables need read-write transaction.
 		 */
 		IsolationLevel level = ISOLATION_REPEATABLE_READ;
-		bool readOnly = false;
 		bool deferrable = true;
 
-		if (!pgsql_set_transaction(pgsql, level, readOnly, deferrable))
+		if (!pgsql_set_transaction(pgsql, level, snapshot->isReadOnly, deferrable))
 		{
 			/* errors have already been logged */
 			(void) pgsql_finish(pgsql);
