@@ -1,11 +1,11 @@
--- KEEPALIVE {"lsn":"0/24E53B8","timestamp":"2024-02-02 09:33:28.571791+0000"}
-BEGIN; -- {"xid":498,"lsn":"0/24E53B8","timestamp":"2024-02-02 09:33:28.572206+0000","commit_lsn":"0/24E5820"}
+-- KEEPALIVE {"lsn":"0/24E86B0","timestamp":"2024-07-30 10:43:53.129529+0000"}
+BEGIN; -- {"xid":500,"lsn":"0/24E86B0","timestamp":"2024-07-30 10:43:53.130860+0000","commit_lsn":"0/24E8B18"}
 PREPARE ec9f2790 AS INSERT INTO "public"."rental" ("rental_id", "rental_date", "inventory_id", "customer_id", "return_date", "staff_id", "last_update") overriding system value VALUES ($1, $2, $3, $4, $5, $6, $7);
 EXECUTE ec9f2790["16050","2022-06-01 00:00:00+00","371","291",null,"1","2022-06-01 00:00:00+00"];
 PREPARE 4afa901a AS INSERT INTO "public"."payment_p2022_06" ("payment_id", "customer_id", "staff_id", "rental_id", "amount", "payment_date") overriding system value VALUES ($1, $2, $3, $4, $5, $6);
 EXECUTE 4afa901a["32099","291","1","16050","5.990000","2022-06-01 00:00:00+00"];
-COMMIT; -- {"xid":498,"lsn":"0/24E5820","timestamp":"2024-02-02 09:33:28.572206+0000"}
-BEGIN; -- {"xid":499,"lsn":"0/24E5820","timestamp":"2024-02-02 09:33:28.573105+0000","commit_lsn":"0/24E6908"}
+COMMIT; -- {"xid":500,"lsn":"0/24E8B18","timestamp":"2024-07-30 10:43:53.130860+0000"}
+BEGIN; -- {"xid":501,"lsn":"0/24E8B18","timestamp":"2024-07-30 10:43:53.132222+0000","commit_lsn":"0/24E9BE8"}
 PREPARE 6ee4a968 AS UPDATE "public"."payment_p2022_02" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
 EXECUTE 6ee4a968["11.950000","23757","116","2","14763","11.990000","2022-02-11 03:52:25.634006+00"];
 PREPARE 6ee4a968 AS UPDATE "public"."payment_p2022_02" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
@@ -26,14 +26,14 @@ PREPARE dc973d3c AS UPDATE "public"."payment_p2022_07" SET "amount" = $1 WHERE "
 EXECUTE dc973d3c["11.950000","28814","592","1","3973","11.990000","2022-07-06 12:15:38.928947+00"];
 PREPARE dc973d3c AS UPDATE "public"."payment_p2022_07" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
 EXECUTE dc973d3c["11.950000","29136","13","2","8831","11.990000","2022-07-22 16:15:40.797771+00"];
-COMMIT; -- {"xid":499,"lsn":"0/24E6908","timestamp":"2024-02-02 09:33:28.573105+0000"}
-BEGIN; -- {"xid":500,"lsn":"0/24E6AC8","timestamp":"2024-02-02 09:33:28.573209+0000","commit_lsn":"0/24E6BD8"}
+COMMIT; -- {"xid":501,"lsn":"0/24E9BE8","timestamp":"2024-07-30 10:43:53.132222+0000"}
+BEGIN; -- {"xid":502,"lsn":"0/24E9DA8","timestamp":"2024-07-30 10:43:53.132367+0000","commit_lsn":"0/24E9EB8"}
 PREPARE 9b3560f5 AS DELETE FROM "public"."payment_p2022_06" WHERE "payment_id" = $1 and "customer_id" = $2 and "staff_id" = $3 and "rental_id" = $4 and "amount" = $5 and "payment_date" = $6;
 EXECUTE 9b3560f5["32099","291","1","16050","5.990000","2022-06-01 00:00:00+00"];
 PREPARE 2ca9993d AS DELETE FROM "public"."rental" WHERE "rental_id" = $1;
 EXECUTE 2ca9993d["16050"];
-COMMIT; -- {"xid":500,"lsn":"0/24E6BD8","timestamp":"2024-02-02 09:33:28.573209+0000"}
-BEGIN; -- {"xid":501,"lsn":"0/24E6BD8","timestamp":"2024-02-02 09:33:28.573357+0000","commit_lsn":"0/24E7158"}
+COMMIT; -- {"xid":502,"lsn":"0/24E9EB8","timestamp":"2024-07-30 10:43:53.132367+0000"}
+BEGIN; -- {"xid":503,"lsn":"0/24E9EB8","timestamp":"2024-07-30 10:43:53.132560+0000","commit_lsn":"0/24EA450"}
 PREPARE 6ee4a968 AS UPDATE "public"."payment_p2022_02" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
 EXECUTE 6ee4a968["11.990000","23757","116","2","14763","11.950000","2022-02-11 03:52:25.634006+00"];
 PREPARE 6ee4a968 AS UPDATE "public"."payment_p2022_02" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
@@ -54,23 +54,41 @@ PREPARE dc973d3c AS UPDATE "public"."payment_p2022_07" SET "amount" = $1 WHERE "
 EXECUTE dc973d3c["11.990000","28814","592","1","3973","11.950000","2022-07-06 12:15:38.928947+00"];
 PREPARE dc973d3c AS UPDATE "public"."payment_p2022_07" SET "amount" = $1 WHERE "payment_id" = $2 and "customer_id" = $3 and "staff_id" = $4 and "rental_id" = $5 and "amount" = $6 and "payment_date" = $7;
 EXECUTE dc973d3c["11.990000","29136","13","2","8831","11.950000","2022-07-22 16:15:40.797771+00"];
-COMMIT; -- {"xid":501,"lsn":"0/24E7158","timestamp":"2024-02-02 09:33:28.573357+0000"}
-BEGIN; -- {"xid":503,"lsn":"0/24E3598","timestamp":"2024-04-26 09:36:17.632070+0000","commit_lsn":"0/24E3870"}
+COMMIT; -- {"xid":503,"lsn":"0/24EA450","timestamp":"2024-07-30 10:43:53.132560+0000"}
+BEGIN; -- {"xid":504,"lsn":"0/24EA450","timestamp":"2024-07-30 10:43:53.132806+0000","commit_lsn":"0/24EA728"}
 PREPARE 21a8a4dc AS DELETE FROM "public"."address" WHERE "address_id" = $1 and "address" = $2 and "address2" IS NULL and "district" = $3 and "city_id" = $4 and "postal_code" = $5 and "phone" = $6 and "last_update" = $7;
 EXECUTE 21a8a4dc["1","47 MySakila Drive","Alberta","300","","","2022-02-15 09:45:30+00"];
 PREPARE 21a8a4dc AS DELETE FROM "public"."address" WHERE "address_id" = $1 and "address" = $2 and "address2" IS NULL and "district" = $3 and "city_id" = $4 and "postal_code" = $5 and "phone" = $6 and "last_update" = $7;
 EXECUTE 21a8a4dc["3","23 Workhaven Lane","Alberta","300","","14033335568","2022-02-15 09:45:30+00"];
 PREPARE 3f3ad11 AS UPDATE "public"."address" SET "postal_code" = $1 WHERE "address_id" = $2 and "address" = $3 and "address2" IS NULL and "district" = $4 and "city_id" = $5 and "postal_code" = $6 and "phone" = $7 and "last_update" = $8;
 EXECUTE 3f3ad11["751007","4","1411 Lillydale Drive","QLD","576","","6172235589","2022-02-15 09:45:30+00"];
-COMMIT; -- {"xid":503,"lsn":"0/24E3870","timestamp":"2024-04-26 09:36:17.632070+0000"}
-BEGIN; -- {"xid":502,"lsn":"0/24E7158","timestamp":"2024-02-02 09:33:28.573418+0000","commit_lsn":"0/24E7800"}
+COMMIT; -- {"xid":504,"lsn":"0/24EA728","timestamp":"2024-07-30 10:43:53.132806+0000"}
+BEGIN; -- {"xid":505,"lsn":"0/24EA728","timestamp":"2024-07-30 10:43:53.132942+0000","commit_lsn":"0/24EAAA0"}
+PREPARE a5a12d9d AS INSERT INTO "public"."generated_column_test" ("id", "name", "email") overriding system value VALUES ($1, $2, $3), ($4, $5, $6), ($7, $8, $9);
+EXECUTE a5a12d9d["1","Tiger","tiger@wild.com","2","Elephant","elephant@wild.com","3","Cat","cat@home.net"];
+COMMIT; -- {"xid":505,"lsn":"0/24EAAA0","timestamp":"2024-07-30 10:43:53.132942+0000"}
+BEGIN; -- {"xid":506,"lsn":"0/24EAAA0","timestamp":"2024-07-30 10:43:53.133035+0000","commit_lsn":"0/24EAC40"}
+PREPARE acf3f3fd AS UPDATE "public"."generated_column_test" SET "name" = $1, "greet_hello" = DEFAULT, "greet_hi" = DEFAULT, "time" = DEFAULT, "email" = $2, "table" = DEFAULT, """table""" = DEFAULT, """hel""lo""" = DEFAULT WHERE "id" = $3;
+EXECUTE acf3f3fd["Lion","tiger@wild.com","1"];
+PREPARE acf3f3fd AS UPDATE "public"."generated_column_test" SET "name" = $1, "greet_hello" = DEFAULT, "greet_hi" = DEFAULT, "time" = DEFAULT, "email" = $2, "table" = DEFAULT, """table""" = DEFAULT, """hel""lo""" = DEFAULT WHERE "id" = $3;
+EXECUTE acf3f3fd["Lion","lion@wild.com","1"];
+COMMIT; -- {"xid":506,"lsn":"0/24EAC40","timestamp":"2024-07-30 10:43:53.133035+0000"}
+BEGIN; -- {"xid":507,"lsn":"0/24EAC40","timestamp":"2024-07-30 10:43:53.133070+0000","commit_lsn":"0/24EAD38"}
+PREPARE acf3f3fd AS UPDATE "public"."generated_column_test" SET "name" = $1, "greet_hello" = DEFAULT, "greet_hi" = DEFAULT, "time" = DEFAULT, "email" = $2, "table" = DEFAULT, """table""" = DEFAULT, """hel""lo""" = DEFAULT WHERE "id" = $3;
+EXECUTE acf3f3fd["Kitten","kitten@home.com","3"];
+COMMIT; -- {"xid":507,"lsn":"0/24EAD38","timestamp":"2024-07-30 10:43:53.133070+0000"}
+BEGIN; -- {"xid":508,"lsn":"0/24EAD38","timestamp":"2024-07-30 10:43:53.133095+0000","commit_lsn":"0/24EADB8"}
+PREPARE 586d3754 AS DELETE FROM "public"."generated_column_test" WHERE "id" = $1;
+EXECUTE 586d3754["2"];
+COMMIT; -- {"xid":508,"lsn":"0/24EADB8","timestamp":"2024-07-30 10:43:53.133095+0000"}
+BEGIN; -- {"xid":509,"lsn":"0/24EADB8","timestamp":"2024-07-30 10:43:53.133218+0000","commit_lsn":"0/24EB460"}
 TRUNCATE ONLY "Sp1eCial .Char"."source1testing"
-COMMIT; -- {"xid":502,"lsn":"0/24E7800","timestamp":"2024-02-02 09:33:28.573418+0000"}
-BEGIN; -- {"xid":503,"lsn":"0/24E7800","timestamp":"2024-02-02 09:33:28.573486+0000","commit_lsn":"0/24E7B18"}
+COMMIT; -- {"xid":509,"lsn":"0/24EB460","timestamp":"2024-07-30 10:43:53.133218+0000"}
+BEGIN; -- {"xid":510,"lsn":"0/24EB460","timestamp":"2024-07-30 10:43:53.133337+0000","commit_lsn":"0/24EB778"}
 PREPARE 5fb4b087 AS INSERT INTO "Sp1eCial .Char"."source1testing" ("s0", "s""1") overriding system value VALUES ($1, $2), ($3, $4), ($5, $6), ($7, $8), ($9, $10);
 EXECUTE 5fb4b087["6","1","7","2","8","3","9","4","10","5"];
-COMMIT; -- {"xid":503,"lsn":"0/24E7B18","timestamp":"2024-02-02 09:33:28.573486+0000"}
-BEGIN; -- {"xid":504,"lsn":"0/24E7B18","timestamp":"2024-02-02 09:33:28.573546+0000","commit_lsn":"0/24E7CD8"}
+COMMIT; -- {"xid":510,"lsn":"0/24EB778","timestamp":"2024-07-30 10:43:53.133337+0000"}
+BEGIN; -- {"xid":511,"lsn":"0/24EB778","timestamp":"2024-07-30 10:43:53.133443+0000","commit_lsn":"0/24EB938"}
 PREPARE 67577134 AS UPDATE "Sp1eCial .Char"."source1testing" SET "s""1" = $1 WHERE "s0" = $2;
 EXECUTE 67577134["2","6"];
 PREPARE 67577134 AS UPDATE "Sp1eCial .Char"."source1testing" SET "s""1" = $1 WHERE "s0" = $2;
@@ -81,16 +99,16 @@ PREPARE 67577134 AS UPDATE "Sp1eCial .Char"."source1testing" SET "s""1" = $1 WHE
 EXECUTE 67577134["8","9"];
 PREPARE 67577134 AS UPDATE "Sp1eCial .Char"."source1testing" SET "s""1" = $1 WHERE "s0" = $2;
 EXECUTE 67577134["10","10"];
-COMMIT; -- {"xid":504,"lsn":"0/24E7CD8","timestamp":"2024-02-02 09:33:28.573546+0000"}
-BEGIN; -- {"xid":505,"lsn":"0/24E7CD8","timestamp":"2024-02-02 09:33:28.573562+0000","commit_lsn":"0/24E7D48"}
+COMMIT; -- {"xid":511,"lsn":"0/24EB938","timestamp":"2024-07-30 10:43:53.133443+0000"}
+BEGIN; -- {"xid":512,"lsn":"0/24EB938","timestamp":"2024-07-30 10:43:53.133469+0000","commit_lsn":"0/24EB9A8"}
 PREPARE fddd6a1b AS DELETE FROM "Sp1eCial .Char"."source1testing" WHERE "s0" = $1;
 EXECUTE fddd6a1b["8"];
-COMMIT; -- {"xid":505,"lsn":"0/24E7D48","timestamp":"2024-02-02 09:33:28.573562+0000"}
-BEGIN; -- {"xid":506,"lsn":"0/24E7D48","timestamp":"2024-02-02 09:33:28.573649+0000","commit_lsn":"0/24E7FF8"}
+COMMIT; -- {"xid":512,"lsn":"0/24EB9A8","timestamp":"2024-07-30 10:43:53.133469+0000"}
+BEGIN; -- {"xid":513,"lsn":"0/24EB9A8","timestamp":"2024-07-30 10:43:53.133576+0000","commit_lsn":"0/24EBC58"}
 PREPARE 477f61f7 AS INSERT INTO "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456"."abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" ("abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456", "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456") overriding system value VALUES ($1, $2), ($3, $4), ($5, $6), ($7, $8), ($9, $10);
 EXECUTE 477f61f7["6","1","7","2","8","3","9","4","10","5"];
-COMMIT; -- {"xid":506,"lsn":"0/24E7FF8","timestamp":"2024-02-02 09:33:28.573649+0000"}
-BEGIN; -- {"xid":507,"lsn":"0/24E7FF8","timestamp":"2024-02-02 09:33:28.573765+0000","commit_lsn":"0/24E8360"}
+COMMIT; -- {"xid":513,"lsn":"0/24EBC58","timestamp":"2024-07-30 10:43:53.133576+0000"}
+BEGIN; -- {"xid":514,"lsn":"0/24EBC58","timestamp":"2024-07-30 10:43:53.133728+0000","commit_lsn":"0/24EBFA8"}
 PREPARE c2fc8166 AS UPDATE "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456"."abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" SET "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456" = $1 WHERE "abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" = $2;
 EXECUTE c2fc8166["4","1"];
 PREPARE c2fc8166 AS UPDATE "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456"."abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" SET "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456" = $1 WHERE "abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" = $2;
@@ -111,6 +129,6 @@ PREPARE c2fc8166 AS UPDATE "ABCDEFGHIJKLMNOPQRSTUVWXYZ01234567890123456789012345
 EXECUTE c2fc8166["8","9"];
 PREPARE c2fc8166 AS UPDATE "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456"."abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" SET "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789012345678901234567890123456" = $1 WHERE "abcdefghijklmnopqrstuvwxyz0123456789012345678901234567890123456" = $2;
 EXECUTE c2fc8166["10","10"];
-COMMIT; -- {"xid":507,"lsn":"0/24E8360","timestamp":"2024-02-02 09:33:28.573765+0000"}
--- KEEPALIVE {"lsn":"0/24E8360","timestamp":"2024-02-02 09:33:28.573772+0000"}
--- ENDPOS {"lsn":"0/24E8360"}
+COMMIT; -- {"xid":514,"lsn":"0/24EBFA8","timestamp":"2024-07-30 10:43:53.133728+0000"}
+-- KEEPALIVE {"lsn":"0/24EBFA8","timestamp":"2024-07-30 10:43:53.133742+0000"}
+-- ENDPOS {"lsn":"0/24EBFA8"}
