@@ -63,3 +63,29 @@ delete from address where city_id = 300 and address2 is null;
 update address set postal_code = '751007' where phone = '6172235589' and address2 is null;
 
 commit;
+
+--
+-- Test generated columns insert, update, and delete
+--
+begin;
+insert into generated_column_test(id, name, email) values
+(1, 'Tiger', 'tiger@wild.com'),
+(2, 'Elephant', 'elephant@wild.com'),
+(3, 'Cat', 'cat@home.net');
+commit;
+
+begin;
+update generated_column_test set name = 'Lion'
+where id = 1;
+update generated_column_test set email='lion@wild.com'
+where email = 'tiger@wild.com';
+commit;
+
+begin;
+update generated_column_test set name = 'Kitten', email='kitten@home.com'
+where id = 3;
+commit;
+
+begin;
+delete from generated_column_test where id = 2;
+commit;
